// FoxClocks extension for Mozilla Firefox/Thunderbird/Sunbird
// Copyright (C) 2006 Andy McDonald / www.stemhaus.com
// For licensing terms, please refer to readme.txt in this extension's '.xpi'
// package or its installation directory on your computer.

// AFM - from http://forums.mozillazine.org/viewtopic.php?t=308369

// ====================================================================================
const CI = Components.interfaces, CC = Components.classes, CR = Components.results;

// ====================================================================================
FoxClocks_Engine.classID = Components.ID("88575671-c0d6-4802-8bd5-a12cfd36247a");
FoxClocks_Engine.contractID = "@stemhaus.com/firefox/foxclocks/engine;1";
FoxClocks_Engine.classDescription = "FoxClocks Engine";

// ====================================================================================
// AFM - Event model:
// - only the engine interfaces with preferences; specifically, other components
//   do not receive pref change notifications
// - engine calls methods on the other components directly; it does not broadcast
//   events to other components
// - engine communicates to chrome JS via ("foxclocks") broadcasts only (the only
//   alternative is pref changes, which makes no sense)
// - currently async notifications from other components back to the engine are by
//   by broadcast. Not sure about this one, but certainly other components are not
//   aware of the engine
// - Note that this model is only enforced in components - chrome JS is still strongly
//   coupled to pref change events, and to writing preferences. Todo, one day
// ====================================================================================

// ====================================================================================
function FoxClocks_Engine()
{
	this.componentStarted = false;

	this._logger = null;
	this._prefManager = null;
	this._updateManager = null;
	this._utils = null;
	this._zoneManager = null;
	this._observerService = null;
	
	this._components = [];
}

// ====================================================================================
FoxClocks_Engine.prototype =
{	
	// ====================================================================================
	startup: function()
	{
		this._logger = CC["@stemhaus.com/firefox/foxclocks/logger;1"].getService(CI.nsISupports).wrappedJSObject;
 		this._prefManager = CC["@stemhaus.com/firefox/foxclocks/prefmanager;1"].getService(CI.nsISupports).wrappedJSObject;
  		this._updateManager = CC["@stemhaus.com/firefox/foxclocks/updatemanager;1"].getService(CI.nsISupports).wrappedJSObject;
   		this._utils = CC["@stemhaus.com/firefox/foxclocks/utils;1"].getService(CI.nsISupports).wrappedJSObject;			
  		this._zoneManager = CC["@stemhaus.com/firefox/foxclocks/zonemanager;1"].getService(CI.nsISupports).wrappedJSObject;			
		this._components = [this._logger, this._prefManager, this._updateManager, this._utils, this._zoneManager];

		this._observerService = CC["@mozilla.org/observer-service;1"].getService(CI.nsIObserverService);
					
 		this._prefManager.addPrefObserver("foxclocks.", this);
		this._prefManager.addPrefObserver("extensions." + this._utils.FC_GUID_FOXCLOCKS + ".", this);
		this._observerService.addObserver(this, "foxclocks", false);
		
		this._onComponentStartup();
		
		this.componentStarted = true;
		CC["@mozilla.org/observer-service;1"].getService(CI.nsIObserverService)
			.notifyObservers(this, "foxclocks", "component:startup");
	},
		
	// ====================================================================================
	shutdown: function()
	{			
		this._prefManager.removePrefObserver("foxclocks.", this);
		this._prefManager.removePrefObserver("extensions." + this._utils.FC_GUID_FOXCLOCKS + ".", this);
		this._observerService.removeObserver(this, "foxclocks");
	},
	
	// ====================================================================================
	onObserve: function(aSubject, aTopic, aData)
	{
		// AFM - rather than observe(), which we leave as boilerplate below
		//
	
		// this._logger.log("FoxClocks_Engine::onObserve(): <" + aTopic + ">, <" +
		//	aData + ">", this._logger.INFO);
				
		if (aTopic == "foxclocks")
		{				
			if (aData == "component:startup")
			{
				if (aSubject.wrappedJSObject != this)
					this._onComponentStartup();
			}
			else if (aData == "updatemanager:update-complete")
			{
				this._onUpdateComplete();
			}
		}
		else if (aTopic == "nsPref:changed")
		{
			switch (aData)
			{
				case "foxclocks.logging.tmpfile.enabled": this._enableTmpLog(); break;
				case "foxclocks.logging.level": this._setLogLevel(); break;
				case "foxclocks.data.update.auto.enabled": this._initUpdateManager(); break;
				case "foxclocks.data.update.auto.interval": this._initUpdateManager(); break;
				case "foxclocks.data.update.noserver.updateurl": this._initUpdateManager(); break;
				
				case "extensions." + this._utils.FC_GUID_FOXCLOCKS + ".devel": this._initUpdateManager(); break;
				case "extensions." + this._utils.FC_GUID_FOXCLOCKS + ".data.update.prevupdate":
						this._initUpdateManager(); break;
						
				case "foxclocks.zonepicker.dataurl":
				{		
					if (this._initZonePicker() == true)
						this._observerService.notifyObservers(null, "foxclocks", "engine:zone-picker-changed");
					break;
				}	
			}
		}		
	},

	// ====================================================================================
	_onComponentStartup: function()
	{
		// AFM - make sure that all components on which we depend have started up before we do anything;
		// this model can be implemented in a cascading fashion
		//
		var startCount = 0;
		
		for (var i=0; i < this._components.length; i++)
		{
			if (this._components[i].componentStarted == true)
				startCount++;
		}
		
		var allStarted = (startCount == this._components.length);
		
		// this._logger.log("FoxClocks_Engine::_onComponentStartup(): " + startCount + "/" +
		//	this._components.length + " components started", this._logger.INFO);
			
		if (allStarted == true)
			this._init();
		
		this._logger.log("-FoxClocks_Engine::_onComponentStartup()");
	},
	
	// ====================================================================================
	_init: function(noUpdateManager)
	{
		// AFM - the real initialisation, after we're sure all other components are up
		// There's still stuff in foxclocksoverlay.js that isn't window-specific -
		// should be moved here
		//
		this._setLogLevel(); // ASAP
		this._enableTmpLog();
		
		// AFM - fix bad update check on startup for 2.0.20. Remove revert stuff on release after 2.0.20
		//		
		var revertTo2_0_19Param = this._prefManager.getPref("extensions." + this._utils.FC_GUID_FOXCLOCKS + ".revertto2_0_19");
		var revertTo2_0_19 = (revertTo2_0_19Param != null && revertTo2_0_19Param == true);
		this._logger.log("+FoxClocks_Engine::_init(): revertTo2_0_19: " + revertTo2_0_19);
		
		if (revertTo2_0_19)
		{
			// AFM - bug: will build update url before the zone manager is initialised, so current version info. is null
			//
			if (noUpdateManager != true)
				this._initUpdateManager();
				
			this._zoneManager.initZoneData(this._updateManager.getLocalDataURL());
			this._initZonePicker();			
		}
		else
		{	
			// AFM - local data url is available before update manager is initialised
			//
			this._zoneManager.initZoneData(this._updateManager.getLocalDataURL());
			this._initZonePicker();
		
			if (noUpdateManager != true)
				this._initUpdateManager();
		}
		
		// AFM - can't imagine why we don't do this as almost the first thing. Not touching right now, though -
		// scary change
		//
		this._convertLegacyParams();
		
		// AFM - 20070220: moved diagnostics from _onComponentStartup()
		//
		this._doDiagnostics();
	},
					
	// ====================================================================================
	_initUpdateManager: function()
	{
		var autoUpenabled = this._prefManager.getPref("foxclocks.data.update.auto.enabled");
		
		// AFM - maximum one update a day
		//
		var prevUpdateSecs = this._prefManager.getPref("extensions." +
					this._utils.FC_GUID_FOXCLOCKS + ".data.update.prevupdate");
					
		var isDevelParam = this._prefManager.getPref("extensions." + this._utils.FC_GUID_FOXCLOCKS + ".devel");
		var isDevel = isDevelParam != null && isDevelParam == true;
			
		var autoUpdateIntervalParam = this._prefManager.getPref("foxclocks.data.update.auto.interval");
		
		// AFM - minimum of 1 day, if we're not in devel mode
		//
		var autoUpdateInterval = isDevel ? autoUpdateIntervalParam : Math.max(autoUpdateIntervalParam, 86400);
		
		var noServerUpdateURL = this._prefManager.getPref("foxclocks.data.update.noserver.updateurl");
		if (noServerUpdateURL == "")
			noServerUpdateURL = null;
			
		this._updateManager.init(autoUpenabled, autoUpdateInterval, prevUpdateSecs, isDevel, noServerUpdateURL);
		
		// AFM - the way the GUI is notified of these changes is poor; if the devel param is modified,
		// eg, the GUI doesn't update (because it's not watching the extensions.{ branch
		//
	},
					
	// ====================================================================================
	_initZonePicker: function()
	{
		const BUILTIN_ZONE_PICKER_DATA_URL = "chrome://foxclocks/locale/zonepicker.dat";
		
		this._logger.log("+FoxClocks_Engine::_initZonePicker()");
						
  		var zonePickerDataUrl = this._prefManager.getPref("foxclocks.zonepicker.dataurl");
		
		if (zonePickerDataUrl == "fc-zonepicker-dataurl-builtin")
			zonePickerDataUrl = BUILTIN_ZONE_PICKER_DATA_URL;
	
		// AFM - failure implies non-builtin URL for non-existent/non-readable file. Probably
		//			
		var ret = this._zoneManager.initZonePicker(zonePickerDataUrl);
		if (ret == false && zonePickerDataUrl != BUILTIN_ZONE_PICKER_DATA_URL)
			ret = this._zoneManager.initZonePicker(BUILTIN_ZONE_PICKER_DATA_URL);
		
		if (ret != true)
			this._logger.log("FoxClocks_Engine::_initZonePicker(): can't retrieve builtin Zone Picker data. Giving up",
					this._logger.ERROR);
		
		this._logger.log("-FoxClocks_Engine::_initZonePicker()");			
		return ret;
	},
	
	// ====================================================================================
	_onUpdateComplete: function()
	{
		// AFM - we set the param on completion of any update attempt
		//
		
		this._prefManager.removePrefObserver("extensions." + this._utils.FC_GUID_FOXCLOCKS + ".", this);
		this._prefManager.setPref("extensions." + this._utils.FC_GUID_FOXCLOCKS + ".data.update.prevupdate",
				Math.round(this._updateManager.lastUpdateDate.getTime()/1000));
		this._prefManager.addPrefObserver("extensions." + this._utils.FC_GUID_FOXCLOCKS + ".", this);

		if (this._updateManager.lastUpdateResult == "OK_NEW")
		{
			this._logger.log("-FoxClocks_Engine::_onUpdateComplete(): reinitialising on new data");
			this._init(true); // do not re-init update manager
		}
			
		this._observerService.notifyObservers(this._updateManager, "foxclocks", "engine:zone-data-update-complete");	
	},
	
	// ====================================================================================
	_setLogLevel: function()
	{
		this._logger.setLevel(this._prefManager.getPref("foxclocks.logging.level"));
	},

	// ====================================================================================
	_enableTmpLog: function()
	{
		this._logger.enableTmpLog(this._prefManager.getPref("foxclocks.logging.tmpfile.enabled"));
	},	
	
	// ====================================================================================
	_convertLegacyParams : function()
	{
		// AFM - from old-style numeric id-based foxclocks.zones.indexes.* params to human-readable
		// id-based single param foxclocks.zones
		//
		
		this._logger.log("+FoxClocks_Engine::_convertLegacyParams()");

		var prefService = CC["@mozilla.org/preferences-service;1"].
				getService(CI.nsIPrefBranch);
						
		// AFM - essentially rename this param - params starting 'foxclocks.' now relate to true
		// configuration, and can be imported/exported
		//	
		var oldPrevRunVersionParam = this._prefManager.getPref("foxclocks.prevrun.version");
		
		prefService.QueryInterface(CI.nsIPrefService);
		if (oldPrevRunVersionParam != null)
		{
			this._prefManager.setPref("extensions." + this._utils.FC_GUID_FOXCLOCKS + ".prevrun.version",
					oldPrevRunVersionParam);
			prefService.getBranch("foxclocks.prevrun.version").deleteBranch("");
		}
		
		// AFM - other old params
		//		
		var zoneIndexesPref = this._prefManager.getPref("foxclocks.zones.indexes");
		var locationDataPref = this._prefManager.getPref("foxclocks.zones.indexes.locationdata");
		var showClockPref = this._prefManager.getPref("foxclocks.zones.indexes.showclock");
					
		prefService.getBranch("foxclocks.zones.indexes").deleteBranch("");
		prefService.getBranch("foxclocks.firstrun").deleteBranch("");
		prefService.getBranch("foxclocks.updateinterval").deleteBranch("");
		
		if (zoneIndexesPref == null || locationDataPref == null || showClockPref == null)
		{
			this._logger.log("-FoxClocks_Engine::_convertLegacyParams(): no legacy data - nothing to do");
			return;
		}

		this._logger.log("FoxClocks_Engine::_convertLegacyParams(): foxclocks.zones.indexes <" + zoneIndexesPref + ">", this._logger.INFO);
		this._logger.log("FoxClocks_Engine::_convertLegacyParams(): foxclocks.zones.indexes.locationdata <" + locationDataPref + ">", this._logger.INFO);
		this._logger.log("FoxClocks_Engine::_convertLegacyParams(): foxclocks.zones.indexes.showclock <" + showClockPref +">", this._logger.INFO);
			
		var zoneIndexes = zoneIndexesPref.split(',');
		if (zoneIndexes.length == 1 && zoneIndexes[0] == "") zoneIndexes = [];
			
		var locationData = locationDataPref.split("|");
		if (locationData.length == 1 && locationData[0] == "") locationData = [];
			
		var showClockData = showClockPref.split(',');
		if (showClockData.length == 1 && showClockData[0] == "") showClockData = [];
				
		var zonePrefsString = "";
		var isFirstIndex = true;
		
 		var stringBundleService = CC["@mozilla.org/intl/stringbundle;1"].getService(CI.nsIStringBundleService);
		var legacyData = stringBundleService.createBundle("chrome://foxclocks/content/zones-legacy.dat");
					
		for (var i = 0; i < zoneIndexes.length && i < locationData.length && i < showClockData.length; i++)
		{				
			var zoneId = null;
			
			try { zoneId = legacyData.GetStringFromName("zone." + zoneIndexes[i]); }
			catch (ex)
			{
				this._logger.log("FoxClocks_Engine::_convertLegacyParams(): coudn't find zone id for legacy zone id <" +
					zoneIndexes[i] + "> - skipping", this._logger.WARN);
				continue;
			}
			
			var zone = this._zoneManager.getZones()[zoneId];
			if (zone == null)
			{
				this._logger.log("FoxClocks_Engine::_convertLegacyParams(): couldn't find zone data for zone id <" +
					zoneId + "> - skipping", this._logger.WARN);
				continue;
			}
			
			var locationDataArray = locationData[i].split("%");
			var locationName = locationDataArray[0];
			var latitude = locationDataArray[1] != "" ? locationDataArray[1] : null;
			var longitude = locationDataArray[2] != "" ? locationDataArray[2] : null;	
			
			var location = this._zoneManager.createLocation(zone, locationName, latitude, longitude);
		
			var styledLocationData = this._zoneManager.createStyledLocationData(location);
			styledLocationData.showClock_statusbar = showClockData[i] == 1;

			if (isFirstIndex)
				isFirstIndex = false;
			else
				zonePrefsString += this._utils.FC_LOCATION_DELIM;
							
			zonePrefsString += styledLocationData.toZoneParam();
		}
		
		this._prefManager.setPref("foxclocks.zones", zonePrefsString);
		
		this._logger.log("-FoxClocks_Engine::_convertLegacyParams(): foxclocks.zones <" +
			zonePrefsString + ">", this._logger.INFO);
	},
	
	// ====================================================================================
	_doDiagnostics: function()
	{
		try
		{
			var appInfo = this._utils.getAppInfo();
			var date = new Date();
			var osString = "nsIXULRuntime" in CI ?
				CC["@mozilla.org/xre/app-info;1"].getService(CI.nsIXULRuntime).OS :
				CC["@mozilla.org/network/protocol;1?name=http"].getService(CI.nsIHttpProtocolHandler).oscpu;
			var localeString = this._prefManager.getPref("general.useragent.locale");
			var localeService = CC["@mozilla.org/intl/nslocaleservice;1"].getService(CI.nsILocaleService);
			var systemLocale = localeService.getSystemLocale().getCategory("NSILOCALE_CTYPE");
			var appLocale = this._prefManager.getPref("general.useragent.locale");
			var dataSource = this._zoneManager.dataSource;
	
 			var extensionsString = "";
 			if ("@mozilla.org/extensions/manager;1" in CC && "nsIExtensionManager" in CI) {
				var em = CC["@mozilla.org/extensions/manager;1"].getService(CI.nsIExtensionManager);
				var addons = em.getItemList(CI.nsIUpdateItem.TYPE_EXTENSION, { });
				for (var i = 0; i < addons.length; i++)
				{
					var addon = addons[i];
					extensionsString += addon.name + " (" + addon.version + ")";

					if (i < addons.length - 1)
						extensionsString += ", ";
				}
			}		
			this._logger.log("DIAGNOSTICS:\n" + 
				"OS: " + osString + " (" + systemLocale + ")\n" +
				"System local time: " + date.toString() + ". Offset is " + date.getTimezoneOffset() + " mins\n" +
				"Application: " + appInfo.appName + " " + appInfo.appVersion + " (" + appLocale + ")\n" + 
				"Extensions: " + extensionsString + "\n" +
				"FoxClocks version: " + this._utils.getFoxClocksVersion() + "\n" + 
				"FoxClocks time zone data: " + dataSource.name + " " + dataSource.version + " (" + dataSource.date +
				"). Auto-update " + (this._updateManager.nextUpdateDate != null ? "enabled" : "disabled")

				, this._logger.INFO);
		}
		catch (ex)
		{
			this._logger.log("FoxClocks_Engine::_doDiagnostics(): diagnostics failed: " + ex, this._logger.ERROR);
		} 		

		this._logger.log("FoxClocks_Engine::_doDiagnostics(): isFirefox15Up " + this._utils.isFirefox15Up); 
	},
			
	// ====================================================================================
	// AFM - nsISupports
	QueryInterface: function(aIID)
	{
		if( aIID.equals(CI.nsISupports) ||
			aIID.equals(CI.nsIClassInfo) ||
			aIID.equals(CI.nsIObserver))
		{
			return this;
		}
		
		throw CR.NS_ERROR_NO_INTERFACE;
	},
  
	// ====================================================================================
	// AFM -  nsIClassInfo  
	getInterfaces: function(aCount)
	{			
		var ifaces = new Array();
		ifaces.push(CI.nsISupports);
		ifaces.push(CI.nsIClassInfo);
		ifaces.push(CI.nsIObserver);
		aCount.value = ifaces.length;
		return ifaces;
	},
  
	// ====================================================================================
	// AFM - nsIClassInfo  
	getHelperForLanguage: function(aLanguage) { return null; },
	get contractID() { return FoxClocks_Engine.contractID; },
	get classID() { return FoxClocks_Engine.classID; },
	get classDescription() { return FoxClocks_Engine.classDescription; },
	get implementationLanguage() { return CI.nsIProgrammingLanguage.JAVASCRIPT; },
	get flags() { return CI.nsIClassInfo.SINGLETON; },
  
	// ====================================================================================
	// AFM - nsIObserver
	observe: function(aSubject, aTopic, aData)
	{
		switch(aTopic)
		{				
			case "xpcom-startup":
				// this is run very early, right after XPCOM is initialized, but before
				// user profile information is applied. Register ourselves as an observer
				// for 'profile-after-change' and 'quit-application'
				//
				var obsSvc = CC["@mozilla.org/observer-service;1"].getService(CI.nsIObserverService);		
				obsSvc.addObserver(this, "profile-after-change", false);
				obsSvc.addObserver(this, "quit-application", false);
				break;
			
			case "profile-after-change":
				// This happens after profile has been loaded and user preferences have been read.
				// startup code here 				
				this.startup();
				break;
			
			case "quit-application":
				this.shutdown();
				var obsSvc = CC["@mozilla.org/observer-service;1"].getService(CI.nsIObserverService);
				obsSvc.removeObserver(this, "profile-after-change");
				obsSvc.removeObserver(this, "quit-application"); 	
				var logService = CC["@stemhaus.com/firefox/foxclocks/logger;1"].getService(CI.nsISupports).wrappedJSObject;
				logService.log("shutdown: " + gCatContractID);				
				break;
		
			default:
				this.onObserve(aSubject, aTopic, aData);
		}
	},
	
	// ====================================================================================
	// AFM - http://www.mozilla.org/scriptable/js-components-status.html
	//
	get wrappedJSObject() { return this; }
};

// ====================================================================================
// constructors for objects we want to XPCOMify
//
var gXpComObjects = [FoxClocks_Engine];
var gCatObserverName = FoxClocks_Engine.classDescription; // can be anything
var gCatContractID = FoxClocks_Engine.contractID;

// **********
// AFM - generic registration code below. Useful URL: http://www.mozilla.org/projects/xpcom/book/cxc/html/weblock.html
// **********

// ====================================================================================
function NSGetModule(compMgr, fileSpec)
{
	gModule._catObserverName = gCatObserverName;
	gModule._catContractId = gCatContractID;
	
	for (var i in gXpComObjects)
		gModule._xpComObjects[i] = new FactoryHolder(gXpComObjects[i]);
		
	return gModule;
}

// ====================================================================================
function FactoryHolder(aObj)
{
	this.classID = aObj.classID;
	this.contractID = aObj.contractID;
	this.className  = aObj.classDescription;
	this.factory =
	{
		createInstance: function(aOuter, aIID)
		{
			if (aOuter)
				throw CR.NS_ERROR_NO_AGGREGATION;
				
			return (new this.constructor).QueryInterface(aIID);
		}
	};
	
	this.factory.constructor = aObj;
}

// ====================================================================================
var gModule =
{
	_xpComObjects: {},
	_catObserverName: null,
	_catContractId: null,
	
	// ====================================================================================
	registerSelf: function(aComponentManager, aFileSpec, aLocation, aType)
	{
		aComponentManager.QueryInterface(CI.nsIComponentRegistrar);
		for (var key in this._xpComObjects)
		{
			var obj = this._xpComObjects[key];
			aComponentManager.registerFactoryLocation(obj.classID, obj.className,
			obj.contractID, aFileSpec, aLocation, aType);
		}
		
		var catman = CC["@mozilla.org/categorymanager;1"].getService(CI.nsICategoryManager);
		catman.addCategoryEntry("xpcom-startup", this._catObserverName, "" + this._catContractId, true, true);
		catman.addCategoryEntry("xpcom-shutdown", this._catObserverName, "" + this._catContractId, true, true);
	},

	// ====================================================================================
	unregisterSelf: function(aCompMgr, aFileSpec, aLocation)
	{
		var catman = CC["@mozilla.org/categorymanager;1"].getService(CI.nsICategoryManager);
		catman.deleteCategoryEntry("xpcom-startup", this._catObserverName, true);
		catman.deleteCategoryEntry("xpcom-shutdown", this._catObserverName, true)
				
		aComponentManager.QueryInterface(CI.nsIComponentRegistrar);
		for (var key in this._xpComObjects)
		{
			var obj = this._xpComObjects[key];
			aComponentManager.unregisterFactoryLocation(obj.classID, aFileSpec);
		}
	},

	// ====================================================================================
	getClassObject: function(aComponentManager, aCID, aIID)
	{
		if (!aIID.equals(CI.nsIFactory))
			throw CR.NS_ERROR_NOT_IMPLEMENTED;
		
		for (var key in this._xpComObjects)
		{
			if (aCID.equals(this._xpComObjects[key].classID))
				return this._xpComObjects[key].factory;
		}
	
		throw CR.NS_ERROR_NO_INTERFACE;
	},

	// ====================================================================================
	canUnload: function(aComponentManager) { return true; }
};