// FoxClocks extension for Mozilla Firefox/Thunderbird/Sunbird
// Copyright (C) 2006 Andy McDonald / www.stemhaus.com
// For licensing terms, please refer to readme.txt in this extension's '.xpi'
// package or its installation directory on your computer.

// AFM - from http://forums.mozillazine.org/viewtopic.php?t=308369

// ====================================================================================
const CI = Components.interfaces, CC = Components.classes, CR = Components.results;
const FC_DEGREE_SYMBOL = "\u00b0";

// ====================================================================================
function FoxClocks_Zone(zoneNode)
{
	this.dl_to_string = "";
	this.dl_name = "";
	this.dl_to = "";
	this.dl_start_gmt = null;
	this.dl_end_gmt = null;
	this.locdName = "TODO";
	
	// AFM - no error checking - will throw
	//
	
	var utils = CC["@stemhaus.com/firefox/foxclocks/utils;1"].getService(CI.nsISupports).wrappedJSObject;	
	this.id = zoneNode.getAttribute("id");

	var standardNode = utils.getFirstEltByTagAsNode(zoneNode, "StandardTime");
	
	this.st_to_string = standardNode.getAttribute("offset");
	this.st_name = standardNode.getAttribute("tzname");
		
	this.st_to = FoxClocks_Zone.timeZoneStringToMinOffset(this.st_to_string);
	
	var daylightNodes = zoneNode.getElementsByTagName("DaylightTime");
	if (daylightNodes.length == 1)
	{
		var daylightNode = daylightNodes.item(0);
		daylightNode.QueryInterface(CI.nsIDOMElement);
		
		this.dl_to_string = daylightNode.getAttribute("offset");
		this.dl_name = daylightNode.getAttribute("tzname");
		var startTime = utils.getFirstEltByTagAsNode(daylightNode, "Start").firstChild.nodeValue;
		var endTime = utils.getFirstEltByTagAsNode(daylightNode, "End").firstChild.nodeValue;
		
		this.dl_to = FoxClocks_Zone.timeZoneStringToMinOffset(this.dl_to_string);
		this.dl_start_gmt = FoxClocks_Zone.dateStringToGmtDate(startTime);
		this.dl_end_gmt = FoxClocks_Zone.dateStringToGmtDate(endTime);
	}
	
	// AFM - lat and long - convert e.g. +0522200 to 52.366667 - no rounding
	//
	var def_latitude = null;
	var def_longitude = null;
	
	var coordsNodes = zoneNode.getElementsByTagName("Coordinates");
	if (coordsNodes.length == 1)
	{
		var coordsNode = coordsNodes.item(0);
		coordsNode.QueryInterface(CI.nsIDOMElement);
		
		var latString = coordsNode.getAttribute("latitude");
		var longString = coordsNode.getAttribute("longitude");
	
		def_latitude = Number(latString.substr(1, 3)) + Number(latString.substr(4, 2))/60 + Number(latString.substr(6, 2))/3600;
		if (latString[0] == "-") def_latitude = -1 * def_latitude;
	
		def_longitude = Number(longString.substr(1, 3)) + Number(longString.substr(4, 2))/60 + Number(longString.substr(6, 2))/3600;
		if (longString[0] == "-") def_longitude = -1 * def_longitude;
		
		/*
		if (latString == "+0353916")
		{
			var logService = CC["@stemhaus.com/firefox/foxclocks/logger;1"].getService(CI.nsISupports).wrappedJSObject;
			logService.log("-FoxClocks_Zone::FoxClocks_Zone(): zone id <" + this.id + ">: " + def_latitude);
		}
		*/
	}
	
	this.defaultLocation = new FoxClocks_Location(null, null, def_latitude, def_longitude);
}

// ====================================================================================
FoxClocks_Zone.prototype = 
{	
	// ====================================================================================
	currName : function(date)
	{
		return this.isDST(date) ? this.dl_name : this.st_name;
	},
	
	// ====================================================================================
	currOffsetString : function(date)
	{
		return this.isDST(date) ? this.dl_to_string: this.st_to_string;
	},
	
	// ====================================================================================
	currDisplayDate : function(date)
	{
		var offsetMillis = null;
		
		if (this.st_to != null)
			offsetMillis = (this.isDST(date) ? this.dl_to: this.st_to)*1000*60;
		else
			offsetMillis = -1 * date.getTimezoneOffset()*1000*60; // local time
	
		// AFM - critical to understand that this date is for display ONLY. Basically you should
		// be looking at getUTCHours() etc ONLY
		//
		return new Date(date.getTime() + offsetMillis);
	},
	
	// ====================================================================================
	isDST : function(date)
	{
		// AFM - note that the the DST start date is considered standard;
		// the DST end date is considered dst. Note that Date.getTimezoneOffset()
		// considers DST start to be dst. Basically don't use isDST() and Date.getTimezoneOffset() together...
		//	
		if (this.dl_start_gmt == null || this.dl_end_gmt == null)
			return false;
		else if (this.dl_start_gmt < this.dl_end_gmt)
			return (this.dl_start_gmt < date && date <= this.dl_end_gmt);
		else // southern hemisphere
			return !(this.dl_end_gmt < date && date <= this.dl_start_gmt);
	}
}
	
// ====================================================================================
FoxClocks_Zone.dateStringToGmtDate = function(dateString)
{
	// AFM - static utility function. No regexp for expected string...
	// dateString eg: 2009-10-21T02:30:00-05:00
	//
	if (dateString == "")
		return null;
			
	var zoneMinOffset = FoxClocks_Zone.timeZoneStringToMinOffset(dateString.substr(19, 5));
	
	var dateTimeArray = dateString.split("T");
	var datePart = dateTimeArray[0];
			
	var date = new Date();
	date.setUTCFullYear(datePart.substr(0, 4), datePart.substr(5, 2) - 1, datePart.substr(8, 2));
		
	if (dateTimeArray.length == 2)
	{	
		var timePart = dateTimeArray[1];
		date.setUTCHours(timePart.substr(0, 2), timePart.substr(3, 2) - zoneMinOffset, timePart.substr(6, 2));
	}
	else
	{
		date.setUTCHours(0, -1 * zoneMinOffset, 0);
	}
		
	return date;
}

// ====================================================================================
FoxClocks_Zone.timeZoneStringToMinOffset = function(timeZoneString)
{					
	// AFM - convert e.g. timeZoneString +01:00 to +60 - static utility funtion
		
	var zoneMinOffset = 60 * Number(timeZoneString.substr(1, 2)) + Number(timeZoneString.substr(4, 2));
	if (timeZoneString.substr(0, 1) == '-')
		zoneMinOffset = -1 * zoneMinOffset;
		
	return zoneMinOffset;
}

// ====================================================================================
// FoxClocks_Location
// ====================================================================================
function FoxClocks_Location(zone, name, lat, lng)
{
	this.zone = zone;
	this.name = name;
	
	this._latitude = null;
	this._lat_degs = 0;
	this._lat_mins = 0;
	this._lat_secs = 0;
	
	this._longitude = null;
	this._long_degs = 0;
	this._long_mins = 0;
	this._long_secs = 0;
		
	this.setLatitude(lat);
	this.setLongitude(lng);
}

// ====================================================================================
FoxClocks_Location.northAsLocaleString = null;
FoxClocks_Location.southAsLocaleString = null;
FoxClocks_Location.eastAsLocaleString = null;
FoxClocks_Location.westAsLocaleString = null;

// ====================================================================================
FoxClocks_Location.prototype =
{
	// ====================================================================================
	getLatitude : function() { return this._latitude; },
	getLongitude : function() { return this._longitude; },
		
	// ====================================================================================
	setLatitude : function(lat)
	{
		// AFM - decimal degrees - 5dp - covers seconds
		//
		// AFM - weak typing: emptyString == 0 is true, and zero == "" is true. lat, lng
		// are sometimes strings, sometimes numbers
		//
		if (lat != null && ((typeof(lat) == "string" && lat != "") || typeof(lat) == "number"))
		{
			this._latitude = Math.round(lat*100000)/100000;
			
			var absLat = Math.abs(this._latitude);
			
			this._lat_degs = Math.floor(absLat);
			this._lat_mins = Math.floor(60 * (absLat - this._lat_degs));
			this._lat_secs = Math.round(3600 * (absLat - this._lat_degs - this._lat_mins/60));
			
			// AFM - may round up to 60
			//
			if (this._lat_secs == 60)
			{
				this._lat_mins++;
				this._lat_secs = 0;
			}	
		}
		else
		{
			this._latitude = null;
			this._lat_degs = 0;
			this._lat_mins = 0;
			this._lat_secs = 0;
		}
		
		/*
		if (this.zone.id == "Africa/Dakar")
		{
			var logService = CC["@stemhaus.com/firefox/foxclocks/logger;1"].getService(CI.nsISupports).wrappedJSObject;
			logService.log("FoxClocks_Location::setLatitude(): deg " + this._lat_degs + " min " + this._lat_mins +
				" sec " + this._lat_secs);
		}
		*/				
	},

	// ====================================================================================
	setLongitude : function(lng)
	{
		if (lng != null && ((typeof(lng) == "string" && lng != "") || typeof(lng) == "number"))
		{
			this._longitude = Math.round(lng*100000)/100000;

			var absLong = Math.abs(this._longitude);
			
			this._long_degs = Math.floor(absLong);
			this._long_mins = Math.floor(60 * (absLong - this._long_degs));
			this._long_secs = Math.round(3600 * (absLong - this._long_degs - this._long_mins/60));
			
			// AFM - may round up to 60
			//
			if (this._long_secs == 60)
			{
				this._long_mins++;
				this._long_secs = 0;
			}	
		}
		else
		{
			this._longitude = null;
			this._long_degs = 0;
			this._long_mins = 0;
			this._long_secs = 0;
		}
		
		/*
		if (this.zone.id == "Africa/Dakar")
		{
			var logService = CC["@stemhaus.com/firefox/foxclocks/logger;1"].getService(CI.nsISupports).wrappedJSObject;
			logService.log("FoxClocks_Location::setLongitude(): deg " + this._long_degs + " min " + this._long_mins +
				" sec " + this._long_secs);
		}
		*/	
	},
	
	// ====================================================================================
	latitudeAsLocaleString : function()
	{
		// AFM - convert e.g. 52.367 to 52{deg. symbol}22' North. Not displaying seconds - ugly
		//
		var latitudeAsString = "";
		if (this._latitude != null)
		{
			var utilsService = CC["@stemhaus.com/firefox/foxclocks/utils;1"]
				.getService(CI.nsISupports).wrappedJSObject;
							
			latitudeAsString = this.latitudeDegrees() + FC_DEGREE_SYMBOL + utilsService.asTwoDigit(this.latitudeMins()) + "'";
			// latitudeAsString += utilsService.asTwoDigit(this.latitudeSecs()) + '"';
			latitudeAsString += " " + (this.latitudeIsNorth() ? FoxClocks_Location.northAsLocaleString : FoxClocks_Location.southAsLocaleString);
		}
		
		return latitudeAsString;
	},

	// ====================================================================================
	longitudeAsLocaleString : function()
	{
		// AFM - convert e.g. -52.367 to 52{deg. symbol}22' West. Not displaying seconds - ugly
		//
		var longitudeAsString = "";
		if (this._longitude != null)
		{
			var utilsService = CC["@stemhaus.com/firefox/foxclocks/utils;1"]
				.getService(CI.nsISupports).wrappedJSObject;
				
			longitudeAsString = this.longitudeDegrees() + FC_DEGREE_SYMBOL + utilsService.asTwoDigit(this.longitudeMins()) + "'";
			// longitudeAsString += utilsService.asTwoDigit(this.longitudeSecs()) + '"';
			longitudeAsString += " " + (this.longitudeIsEast() ? FoxClocks_Location.eastAsLocaleString : FoxClocks_Location.westAsLocaleString);
		}
		
		return longitudeAsString;
	},
	
	// ====================================================================================
	latitudeIsNorth : function() { return this._latitude != null && this._latitude >= 0; },
	latitudeDegrees : function() { return this._lat_degs; },
	latitudeMins : function() { return this._lat_mins; },
	latitudeSecs : function() { return this._lat_secs; },
	longitudeIsEast : function() { return this._longitude != null && this._longitude >= 0; },
	longitudeDegrees : function() { return this._long_degs; },
	longitudeMins : function() { return this._long_mins; },
	longitudeSecs : function() { return this._long_secs; }
}

// ====================================================================================
// StyledLocationData
//
// AFM - should not be in zonemanager.js at all... And should have a sensible name
// ====================================================================================
function StyledLocationData(location)
{
	this.location = location;
	this.showClock_statusbar = true;
	this.showClock_statusbarTooltip = true;
	this.bold = false;
	this.italic = false;
	this.underline = false;
	this.colour = "";
	this.altColour_enabled = false;
	this.altColour = "";
	this.altColour_startTime = 540; // 9am
	this.altColour_endTime = 1020; // 5pm
}

// ====================================================================================
StyledLocationData.prototype =
{
	// ====================================================================================
	populateFromZoneParam : function(zoneParam)
	{
		var logger = CC["@stemhaus.com/firefox/foxclocks/logger;1"]
				.getService(CI.nsISupports).wrappedJSObject;
		var zoneManager = CC["@stemhaus.com/firefox/foxclocks/zonemanager;1"]
				.getService(CI.nsISupports).wrappedJSObject;
								
		var zoneParamArray = zoneParam.split("%");
		if (zoneParamArray == "" || zoneParamArray.length < 4) // magic number - yuk
		{
			logger.log("StyledLocationData::populateFromZoneParam(): insufficient data", logger.ERROR);
			return false;
		}
				
		var zoneId = zoneParamArray[0];
		var locationName = zoneParamArray[1];
		var latitude = zoneParamArray[2] != "" ? zoneParamArray[2] : null;
		var longitude = zoneParamArray[3] != "" ? zoneParamArray[3] : null;
							
		var zone = zoneManager.getZones()[zoneId];
		if (zone == null)
		{
			logger.log("StyledLocationData::populateFromZoneParam(): could not find zone with id <" +
				zoneId + ">", logger.ERROR);
			return false;
		}
		
		// AFM - we persist a locationName matching the zone's default location as "",
		// so that in the case that someone switches locales, their locations will be renamed nicely
		// More significantly, because of this the default foxclocks.zones param doesn't need to be localised
		//
		if (locationName == "")
			locationName = zone.defaultLocation.name;

		this.location = new FoxClocks_Location(zone, locationName, latitude, longitude);

		if (zoneParamArray.length > 4) this.showClock_statusbar = zoneParamArray[4] == 1;
		if (zoneParamArray.length > 5) this.showClock_statusbarTooltip = zoneParamArray[5] == 1;
		if (zoneParamArray.length > 6) this.bold = zoneParamArray[6] == 1;
		if (zoneParamArray.length > 7) this.italic = zoneParamArray[7] == 1;
		if (zoneParamArray.length > 8) this.underline = zoneParamArray[8] == 1;
		if (zoneParamArray.length > 9) this.colour = zoneParamArray[9];
		if (zoneParamArray.length > 10) this.altColour_enabled = zoneParamArray[10] == 1;
		if (zoneParamArray.length > 11) this.altColour = zoneParamArray[11];			
		if (zoneParamArray.length > 12) this.altColour_startTime = Number(zoneParamArray[12]);
		if (zoneParamArray.length > 13) this.altColour_endTime = Number(zoneParamArray[13]);
		
		return true;
	},

	// ====================================================================================
	toZoneParam : function()
	{	
		return this.location.zone.id + "%" +
				(this.location.name == this.location.zone.defaultLocation.name ? "" : this.location.name) + "%" +
				(this.location.getLatitude() != null ? this.location.getLatitude() : "") + "%" +
				(this.location.getLongitude() != null ? this.location.getLongitude() : "") + "%" +
				(this.showClock_statusbar ? "1" : "0") + "%" +
				(this.showClock_statusbarTooltip ? "1" : "0") + "%" +
				(this.bold ? "1" : "0") + "%" +
				(this.italic ? "1" : "0") + "%" +
				(this.underline ? "1" : "0") + "%" +
				this.colour + "%" + 
				(this.altColour_enabled ? "1" : "0") + "%" +
				this.altColour + "%" +
				this.altColour_startTime + "%" +
				this.altColour_endTime;
	}
}

// ====================================================================================
// FoxClocks_ZoneManager
// ====================================================================================
FoxClocks_ZoneManager.classID = Components.ID("3c184c60-5496-11db-8373-b622a1ef5492");
FoxClocks_ZoneManager.contractID = "@stemhaus.com/firefox/foxclocks/zonemanager;1";
FoxClocks_ZoneManager.classDescription = "FoxClocks Zone Manager Service";

// ====================================================================================
function FoxClocks_ZoneManager()
{
	this.componentStarted = false;

	this.zones = null;
	this.zonePickerArray = null;
	this.dataSource = {id: null, name: null, date: null, version: null};
	this.logger = null;
}

// ====================================================================================
FoxClocks_ZoneManager.prototype =
{	
	// ====================================================================================
	startup: function()
	{
		// ASAP
		//
		this.logger = CC["@stemhaus.com/firefox/foxclocks/logger;1"].getService(CI.nsISupports).wrappedJSObject;

 		var stringBundleService = CC["@mozilla.org/intl/stringbundle;1"].getService(CI.nsIStringBundleService);
		
		// AFM - a bit rubbish
		//
		var hFoxClocksBundle = stringBundleService.createBundle("chrome://foxclocks/locale/foxclocks.properties");

		FoxClocks_Location.northAsLocaleString = hFoxClocksBundle.GetStringFromName("misc.north");
		FoxClocks_Location.southAsLocaleString = hFoxClocksBundle.GetStringFromName("misc.south");
		FoxClocks_Location.eastAsLocaleString = hFoxClocksBundle.GetStringFromName("misc.east");
		FoxClocks_Location.westAsLocaleString = hFoxClocksBundle.GetStringFromName("misc.west");
		
		this.componentStarted = true;
		CC["@mozilla.org/observer-service;1"].getService(CI.nsIObserverService)
			.notifyObservers(this, "foxclocks", "component:startup");				
	},
		
	// ====================================================================================
	shutdown: function()
	{
	},
	
	// ====================================================================================
	onObserve: function(aSubject, aTopic, aData)
	{
	},

	// ====================================================================================
	initZoneData: function(zoneDataURL)
	{
		var utilsService = CC["@stemhaus.com/firefox/foxclocks/utils;1"].getService(CI.nsISupports).wrappedJSObject;
 		var httpRequest = CC["@mozilla.org/xmlextras/xmlhttprequest;1"].createInstance(CI.nsIXMLHttpRequest);

		// AFM - generate zone data
		//
		this.logger.log("FoxClocks_ZoneManager::initZoneData(): zoneDataURL is " + zoneDataURL);		

		httpRequest.open('GET', zoneDataURL, false);
		httpRequest.send(null);
		var doc = httpRequest.responseXML;
		var root = doc.documentElement;
		
		if (root.nodeName == "parsererror")
		{
			this.logger.log("FoxClocks_ZoneManager::initZoneData(): parsererror: " +
					root.firstChild.nodeValue, this.logger.ERROR);
			return false;
		}

		try
		{						
			var sourceNode = utilsService.getFirstEltByTagAsNode(root, "Source");
			var sourceId = sourceNode.getAttribute("id");
			var sourceName = sourceNode.getAttribute("name");
			var sourceVersion = utilsService.getFirstEltByTagAsNode(sourceNode, "Version").firstChild.nodeValue;
			var sourceDate = utilsService.getFirstEltByTagAsNode(sourceNode, "Date").firstChild.nodeValue;
			
			this.logger.log("FoxClocks_ZoneManager::initZoneData(): start, setting data source info");
			
			this.dataSource.id = sourceId;
			this.dataSource.name = sourceName;
			this.dataSource.version = sourceVersion;
			this.dataSource.date = sourceDate;
			
			this.logger.log("FoxClocks_ZoneManager::initZoneData(): end, setting data source info");
		}
		catch (ex)
		{
			this.logger.log("FoxClocks_ZoneManager::initZoneData(): error reading XML Source data: " +
				ex, this.logger.ERROR);
				
			return false;
		}
		
		this.zones = new Array();
		var zoneNodes = root.getElementsByTagName("Zone");
		for (var i=0; i < zoneNodes.length; i++)
		{
			var zoneNode = zoneNodes.item(i);
			zoneNode.QueryInterface(CI.nsIDOMElement);
			
			try
			{
				var zone = new FoxClocks_Zone(zoneNode);
				this.zones[zone.id] = zone;
			}
			catch (ex)
			{
				this.logger.log("FoxClocks_ZoneManager::initZoneData(): error creating zone from DOM: " +
					ex, this.logger.ERROR);
			}
		}
		
		return true;
	},
		
	// ====================================================================================
	initZonePicker: function(zonePickerDataUrl)
	{
		var utilsService = CC["@stemhaus.com/firefox/foxclocks/utils;1"].getService(CI.nsISupports).wrappedJSObject;
 
		this.zonePickerArray = new Array();
															 
		var zoneLines = utilsService.getUriLines(zonePickerDataUrl, "UTF-8");		
		if (zoneLines == null)
		{
			// AFM - failure implies non-builtin URL for non-existent/non-readable file. Probably
			//
			return false;
		}	
			
		// AFM - write locale data on top of zones. THIS DOES NOT SUPPORT MULTIPLE NAMES FOR SAME ZONE
		//
		const FC_REGEXP_ZONEIDPROPERTY = /^(.+?)=(.+)$/;
		 		
		for (var i = 0; i < zoneLines.length; i++)
		{
			var matchDataArray = FC_REGEXP_ZONEIDPROPERTY.exec(zoneLines[i]);
			if (matchDataArray == null) // i.e. we don't have a match
				continue;
				
			var zoneId = matchDataArray[1]; // property key from capturing paren
			var zoneLocaleData = matchDataArray[2]; // property key from capturing paren
			
			// AFM - zoneLocaleData is eg Europe|Oxford[%51.76%-1.26[%DEFAULT]]
			//
			var zoneLocaleDataAsArray = zoneLocaleData.split('%');
			if (zoneLocaleDataAsArray.length != 1 && zoneLocaleDataAsArray.length != 3 &&
					zoneLocaleDataAsArray.length != 4)
			{
				this.logger.log("FoxClocks_ZoneManager::initZonePicker(): cannot generate zone picker item with zoneid <" +
					zoneId + ">: bad locale data - skipping", this.logger.ERROR);

				continue;
			}
								
			var zone = this.zones[zoneId];
			
			if (zone == null)
			{
				// AFM - log this at debug - expected for the moment
				//
				this.logger.log("FoxClocks_ZoneManager::initZonePicker(): cannot generate zone picker item with zoneid <" +
					zoneId + ">: no underlying zone data - skipping");

				continue;
			}
			
			var zoneLocaleName = zoneLocaleDataAsArray[0]; // eg: Europe|London
			var zoneLocaleLat = zoneLocaleDataAsArray.length >= 3 ? zoneLocaleDataAsArray[1] : zone.defaultLocation.getLatitude();
			var zoneLocaleLong = zoneLocaleDataAsArray.length >= 3 ? zoneLocaleDataAsArray[2] : zone.defaultLocation.getLongitude();	
		
			// AFM - take last part of zoneLocaleName - location's name s/b short
			//
			var zoneLocaleNameArray = zoneLocaleName.split("|");
			var zoneLocaleShortName = zoneLocaleNameArray[zoneLocaleNameArray.length - 1]
		
			// AFM - TODO - no - if we don't have a lat/long, we shouldn't just copy in the default location's lat/long,
			// since we don't pick up updates to it. Not changing right now b/c this is a house of cards
			//
			var location = new FoxClocks_Location(zone, zoneLocaleShortName, zoneLocaleLat, zoneLocaleLong);
			
			// AFM - add zp_structure to location for the zone picker. Not going into JS inheritance
			//
			location.zp_structure = zoneLocaleName;
			
			this.zonePickerArray.push(location);
			
			// AFM - we need a localised default location name
			// (we already have def. lat and long). We derive as either the first location
			// in ZP, or the location marked DEFAULT
			//
			if ((zoneLocaleDataAsArray.length >= 4 && zoneLocaleDataAsArray[3] == "DEFAULT")
				|| zone.defaultLocation.name == null)
			{	
				zone.defaultLocation.name = location.name;
				
				// AFM - commented out - see TODO above - horribly broken
				//
				// zone.defaultLocation.setLatitude(location.getLatitude());
				// zone.defaultLocation.setLongitude(location.getLongitude());
			}
		}
		
		return true;
	},
	
	// ====================================================================================
	getZones: function() { return this.zones; },
	getZonePickerArray: function() { return this.zonePickerArray; },
	createLocation: function(zone, locationName, lat, lng) 
		{ return new FoxClocks_Location(zone, locationName, lat, lng); },
	createStyledLocationData: function(location) 
		{ return new StyledLocationData(location); },
				
	// ====================================================================================
	// AFM - nsISupports
	QueryInterface: function(aIID)
	{
		if( aIID.equals(CI.nsISupports) ||
			aIID.equals(CI.nsIClassInfo) ||
			aIID.equals(CI.nsIObserver))
		{
			return this;
		}
		
		throw CR.NS_ERROR_NO_INTERFACE;
	},
  
	// ====================================================================================
	// AFM -  nsIClassInfo  
	getInterfaces: function(aCount)
	{			
		var ifaces = new Array();
		ifaces.push(CI.nsISupports);
		ifaces.push(CI.nsIClassInfo);
		ifaces.push(CI.nsIObserver);
		aCount.value = ifaces.length;
		return ifaces;
	},
  
	// ====================================================================================
	// AFM - nsIClassInfo  
	getHelperForLanguage: function(aLanguage) { return null; },
	get contractID() { return FoxClocks_ZoneManager.contractID; },
	get classID() { return FoxClocks_ZoneManager.classID; },
	get classDescription() { return FoxClocks_ZoneManager.classDescription; },
	get implementationLanguage() { return CI.nsIProgrammingLanguage.JAVASCRIPT; },
	get flags() { return CI.nsIClassInfo.SINGLETON; },
  
	// ====================================================================================
	// AFM - nsIObserver
	observe: function(aSubject, aTopic, aData)
	{	
		switch(aTopic)
		{				
			case "xpcom-startup":
				// this is run very early, right after XPCOM is initialized, but before
				// user profile information is applied. Register ourselves as an observer
				// for 'profile-after-change' and 'quit-application'
				//			
				var obsSvc = CC["@mozilla.org/observer-service;1"].getService(CI.nsIObserverService);
				obsSvc.addObserver(this, "profile-after-change", false);
				obsSvc.addObserver(this, "quit-application", false);
				break;
			
			case "profile-after-change":
				// This happens after profile has been loaded and user preferences have been read.
				// startup code here 				
				this.startup();
				break;
			
			case "quit-application":
				this.shutdown();
				var obsSvc = CC["@mozilla.org/observer-service;1"].getService(CI.nsIObserverService);
				obsSvc.removeObserver(this, "profile-after-change");
				obsSvc.removeObserver(this, "quit-application"); 	
				var logService = CC["@stemhaus.com/firefox/foxclocks/logger;1"].getService(CI.nsISupports).wrappedJSObject;
				logService.log("shutdown: " + gCatContractID);				
				break;
				
			default:
				this.onObserve(aSubject, aTopic, aData);
		}
	},
	
	// ====================================================================================
	// AFM - http://www.mozilla.org/scriptable/js-components-status.html
	//
	get wrappedJSObject() { return this; }
};

// ====================================================================================
// constructors for objects we want to XPCOMify
//
var gXpComObjects = [FoxClocks_ZoneManager];
var gCatObserverName = FoxClocks_ZoneManager.classDescription; // can be anything
var gCatContractID = FoxClocks_ZoneManager.contractID;

// **********
// AFM - generic registration code below. Useful URL: http://www.mozilla.org/projects/xpcom/book/cxc/html/weblock.html
// **********

// ====================================================================================
function NSGetModule(compMgr, fileSpec)
{
	gModule._catObserverName = gCatObserverName;
	gModule._catContractId = gCatContractID;
	
	for (var i in gXpComObjects)
		gModule._xpComObjects[i] = new FactoryHolder(gXpComObjects[i]);
		
	return gModule;
}

// ====================================================================================
function FactoryHolder(aObj)
{
	this.classID = aObj.classID;
	this.contractID = aObj.contractID;
	this.className  = aObj.classDescription;
	this.factory =
	{
		createInstance: function(aOuter, aIID)
		{
			if (aOuter)
				throw CR.NS_ERROR_NO_AGGREGATION;
				
			return (new this.constructor).QueryInterface(aIID);
		}
	};
	
	this.factory.constructor = aObj;
}

// ====================================================================================
var gModule =
{
	_xpComObjects: {},
	_catObserverName: null,
	_catContractId: null,
	
	// ====================================================================================
	registerSelf: function(aComponentManager, aFileSpec, aLocation, aType)
	{
		aComponentManager.QueryInterface(CI.nsIComponentRegistrar);
		for (var key in this._xpComObjects)
		{
			var obj = this._xpComObjects[key];
			aComponentManager.registerFactoryLocation(obj.classID, obj.className,
			obj.contractID, aFileSpec, aLocation, aType);
		}
		
		var catman = CC["@mozilla.org/categorymanager;1"].getService(CI.nsICategoryManager);
		catman.addCategoryEntry("xpcom-startup", this._catObserverName, "" + this._catContractId, true, true);
		catman.addCategoryEntry("xpcom-shutdown", this._catObserverName, "" + this._catContractId, true, true);
	},

	// ====================================================================================
	unregisterSelf: function(aCompMgr, aFileSpec, aLocation)
	{
		var catman = CC["@mozilla.org/categorymanager;1"].getService(CI.nsICategoryManager);
		catman.deleteCategoryEntry("xpcom-startup", this._catObserverName, true);
		catman.deleteCategoryEntry("xpcom-shutdown", this._catObserverName, true)
				
		aComponentManager.QueryInterface(CI.nsIComponentRegistrar);
		for (var key in this._xpComObjects)
		{
			var obj = this._xpComObjects[key];
			aComponentManager.unregisterFactoryLocation(obj.classID, aFileSpec);
		}
	},

	// ====================================================================================
	getClassObject: function(aComponentManager, aCID, aIID)
	{
		if (!aIID.equals(CI.nsIFactory))
			throw CR.NS_ERROR_NOT_IMPLEMENTED;
		
		for (var key in this._xpComObjects)
		{
			if (aCID.equals(this._xpComObjects[key].classID))
				return this._xpComObjects[key].factory;
		}
	
		throw CR.NS_ERROR_NO_INTERFACE;
	},

	// ====================================================================================
	canUnload: function(aComponentManager) { return true; }
};