// FoxClocks extension for Mozilla Firefox/Thunderbird/Sunbird
// Copyright (C) 2006 Andy McDonald / www.stemhaus.com
// For licensing terms, please refer to readme.txt in this extension's '.xpi'
// package or its installation directory on your computer.

// ====================================================================================
function FoxClocks()
{		
	this.zonePicker = null;
	this.watchList = null;
	this.watchListData = new Array();
	this.timeFormatter = null;
	this.inOnZonePickerSelect = false;
	this.hTimer = null;
}
	
// ====================================================================================
FoxClocks.prototype =
{
	// ====================================================================================	
	quickTest : function()
	{
		// const TESTING = true;
		const TESTING = false;
		
		fc_gLogger.log("FoxClocks::quickTest(): enabled: " + TESTING);

		if (TESTING == false)
			return;
			
		try
		{	
			// ====================================================================================
			// ====================================================================================

			fc_gLogger.log("FoxClocks::quickTest(): default");
			fc_gLogger.log("FoxClocks::quickTest(): debug", fc_gLogger.DEBUG);
			fc_gLogger.log("FoxClocks::quickTest(): info", fc_gLogger.INFO);
			fc_gLogger.log("FoxClocks::quickTest(): warn", fc_gLogger.WARN);
			fc_gLogger.log("FoxClocks::quickTest(): error", fc_gLogger.ERROR);
		
			var doc = this.getDOMImpl().createDocument(FC_URL_FOXCLOCKSHOME + "zonepicker", "zonepicker", "2.0");
					
			var root = document.getElementById("fc-zonepicker-treechildren-root");

			for (var i=0; i < root.childNodes.length; i++)
			{
				var child = root.childNodes[i];
				
				if (k < zoneNameAsArray.length - 1) // i.e. is not terminal node
				{		
					var childTreeChildren = document.getElementById(this.getTreeChildrenId(k, zoneNameAsArray));
					if (childTreeChildren == null)
					{
						childTreeChildren = document.createElement("treechildren");
						childTreeChildren.setAttribute("id", this.getTreeChildrenId(k, zoneNameAsArray));
						
						var treeItem1 = document.createElement("treeitem");
						treeItem1.setAttribute("container", "true");
						
						parentTreeChildren.appendChild(treeItem1);
						
						var treeCell1 = document.createElement("treecell");
						treeCell1.setAttribute("label", zoneNameAsArray[k]);
						
						var treeRow1 = document.createElement("treerow");
						
						treeRow1.appendChild(treeCell1);
						treeItem1.appendChild(treeRow1);
						treeItem1.appendChild(childTreeChildren);
					}
				}
				else
				{										
					var treeItem2 = document.createElement("treeitem");
					treeItem2.setAttribute("value", i); // AFM - to allow lookup in this.zonePickerArray
					
					parentTreeChildren.appendChild(treeItem2);
					
					var cellCurrZoneLabel = currLocation.zone.currOffsetString(nowDate) +
						" (" + currLocation.zone.currName(nowDate) + ")";
		
					if (currLocation.zone.isDST(nowDate))
						cellCurrZoneLabel += "*";
					
					var treeCellRegion = document.createElement("treecell");
					treeCellRegion.setAttribute("label", zoneNameAsArray[k]);
		
					var treeCellCurrZone = document.createElement("treecell");
					treeCellCurrZone.setAttribute("label", cellCurrZoneLabel);
		
					var treeRow2 = document.createElement("treerow");					
					treeRow2.appendChild(treeCellRegion);
					treeRow2.appendChild(treeCellCurrZone);
					
					treeItem2.appendChild(treeRow2);
				}
			}
			// ====================================================================================
			// ====================================================================================
			fc_gLogger.log("FoxClocks::quickTest(): success");
		}
		catch (ex)
		{
			alert("quickTest(): exception <" + ex + ">");
		}
	},
		
	// ====================================================================================
	onLoad : function()
	{
		fc_gLogger.log("+FoxClocks::onLoad()");

		setTimeout(this.quickTest, 0, this);
		
		this.zonePicker = document.getElementById("fc-zonepicker");
		this.watchList = document.getElementById("fc-watchlist");
		
		fc_gPrefManager.addPrefObserver("foxclocks.", this);
		fc_gObserverService.addObserver(this, "foxclocks", false);
			
		this.timeFormatter = new FoxClocks_TimeFormatter();
		
		this.setTimerInterval();
		this.setTimeFormat();
		
		var versionString = "FoxClocks " + fc_gUtils.getFoxClocksVersion();
		document.getElementById("fc-foxclocks-version-label").setAttribute("value", versionString);
		
		this.populateWatchlist();
		this.populateZonePicker();
			
		fc_gLogger.log("-FoxClocks::onLoad()");
	},
	
	// ====================================================================================
	onUnload : function()
	{
		if (this.hTimer != null)
			window.clearInterval(this.hTimer);
			
		fc_gPrefManager.removePrefObserver("foxclocks.", this);
		fc_gObserverService.removeObserver(this, "foxclocks");
	},
	
	// AFM - START, EVENTS
	//
	
	// ====================================================================================
	onWatchListKeyPress : function(event)
	{
		if (event.keyCode == KeyEvent.DOM_VK_ENTER || event.keyCode == KeyEvent.DOM_VK_RETURN)
			this.onOpenLocDetailsCmd();
	},

	// ====================================================================================
	onZonePickerSelect : function()
	{
		// AFM - horrible way to disable selecting non-terminal nodes
		//
		
		// AFM - prevent recursion. this.zonePicker.view.selection.selectEventsSuppressed
		// doesn't seem to work inside an event handler - in fact causes recursion.
		// We're called back again when we clear a container's selection, so can't use
		// toggleSelection since we'll reselect the container. Yuk.
		//
		if (this.inOnZonePickerSelect)
			return;
		
		this.inOnZonePickerSelect = true;
	
		fc_gLogger.log("+FoxClocks::onZonePickerSelect()");
		
		var rangeCount = this.zonePicker.view.selection.getRangeCount();
		
		for (var i=0; i < rangeCount; i++)
		{
			var start = {};
			var end = {};
			this.zonePicker.view.selection.getRangeAt(i, start, end);
			for (var c = start.value; c != -1 && c <= end.value; c++)
			{
				var treeItem = this.zonePicker.view.getItemAtIndex(c);

				if (treeItem.getAttribute("container") == "true")
					this.zonePicker.view.selection.clearRange(c, c);
			}
		}
		
		this.setZonePickerStates();
		this.inOnZonePickerSelect = false;
		
		fc_gLogger.log("-FoxClocks::onZonePickerSelect()");
	},
	
	// ====================================================================================
	onOpenZoneDetailsCmd : function()
	{
		var location = this.getSelectedZonePickerLocation();
		if (!location)
			return;
					
		// AFM - ignore return value
		//
		var styledLocationData = fc_gZoneManager.createStyledLocationData(location);
		this.openZoneInfo(styledLocationData, "ZONE");
	},
	
	// ====================================================================================
	onOpenZoneGoogleEarthCmd : function()
	{
		var location = this.getSelectedZonePickerLocation();
		if (!location)
			return;
			
		this.openGoogleEarth(location);
	},
	
	// ====================================================================================
	onAddCmd : function()
	{
		fc_gLogger.log("+FoxClocks::onAddCmd()");
												
		var rangeCount = this.zonePicker.view.selection.getRangeCount();
		var addedLocations = new Array();
		
		var zonePickerArray = fc_gZoneManager.getZonePickerArray();
		
		for (var i=0; i < rangeCount; i++)
		{
			var start = {};
			var end = {};
			this.zonePicker.view.selection.getRangeAt(i, start, end);
			for (var c = start.value; c != -1 && c <= end.value; c++)
			{
				var j = this.zonePicker.view.getItemAtIndex(c).getAttribute("value");
				if (j != null)
				{
					var styledLocationData = fc_gZoneManager.createStyledLocationData(zonePickerArray[j]);
					addedLocations.push(styledLocationData);
				}
			}
		}
		
		this.addLocations(addedLocations);
		this.setWatchlistStates();
		this.setZonesPref();
		this.removeSortAttributes();
				
		fc_gLogger.log("-FoxClocks::onAddCmd()");
	},
	
	// ====================================================================================
	onImportCmd : function()
	{
		var dialogTitle = document.getElementById("fc-dialog-import-title").getAttribute("label");
		var foxClocksFilter = "*." + FC_FOXCLOCKS_SETTINGS_EXTENSION;
		var filterText = document.getElementById("fc-dialog-importexport-filter-label").getAttribute("label") +
			" (" + foxClocksFilter + ")";

		var nsIFilePicker = Components.interfaces.nsIFilePicker;
		var fp = Components.classes["@mozilla.org/filepicker;1"].createInstance(nsIFilePicker);
		fp.init(window, dialogTitle, nsIFilePicker.modeOpen);
		fp.appendFilter(filterText, foxClocksFilter);
		fp.appendFilters(nsIFilePicker.filterAll);
		
		var res = fp.show();
		
		var errorMsg = null;
		if (res != nsIFilePicker.returnOK)
			return;
							
		try
		{
			var inStream = Components.classes["@mozilla.org/network/file-input-stream;1"]
					.createInstance(Components.interfaces.nsIFileInputStream);
			inStream.init(fp.file, -1, -1, 0);

			var doc = new DOMParser().parseFromStream(inStream, "UTF-8", -1, "text/xml");
			
			inStream.close();
			
			var promptText = document.getElementById("fc-import-confirm-label").getAttribute("label");  	
			if (fc_gPromptService.confirm(window, "FoxClocks", promptText) == false)
				return;
			
			if (fc_gPrefManager.xmlToPrefs("foxclocks.", doc) == false)
				errorMsg = "parsererror"; // bit lame - not localised
		} 
		catch(ex) 
		{
			errorMsg = ex.message;
		}
		
		if (errorMsg != null)
		{
			var promptText = document.getElementById("fc-dialog-import-failure").getAttribute("label") + ": " + errorMsg;
			fc_gPromptService.alert(window, "FoxClocks", promptText);
		}
		else
		{				
			var promptText = document.getElementById("fc-dialog-import-success").getAttribute("label");
			FoxClocks_openSimpleInfo("FoxClocks", promptText);
		}
	},

	// ====================================================================================
	onExportCmd : function()
	{
		fc_gLogger.log("+FoxClocks::onExportCmd()");

		var dialogTitle = document.getElementById("fc-dialog-export-title").getAttribute("label");
		var foxClocksFilter = "*." + FC_FOXCLOCKS_SETTINGS_EXTENSION;
		var filterText = document.getElementById("fc-dialog-importexport-filter-label").getAttribute("label") +
			" (" + foxClocksFilter + ")";								

		var nsIFilePicker = Components.interfaces.nsIFilePicker;
		var fp = Components.classes["@mozilla.org/filepicker;1"].createInstance(nsIFilePicker);
		fp.init(window, dialogTitle, nsIFilePicker.modeSave);
		fp.defaultExtension = FC_FOXCLOCKS_SETTINGS_EXTENSION;
		fp.defaultString = "foxclocks";
		fp.appendFilter(filterText, foxClocksFilter);
		fp.appendFilters(nsIFilePicker.filterAll);
		
		var res = fp.show();
		
		if (res == nsIFilePicker.returnOK || res == nsIFilePicker.returnReplace)
		{
			try
			{
				// AFM - well this is quite a mess. https://bugzilla.mozilla.org/show_bug.cgi?id=318086
				// Would like to be able to use nsIDOMSerializer.serializeToStream()
				//
				var doc = fc_gPrefManager.prefsToXml("foxclocks.", FC_URL_FOXCLOCKSHOME + "prefs",
									"prefs", fc_gUtils.getFoxClocksVersion());
					
				var serializer = Components.classes["@mozilla.org/xmlextras/xmlserializer;1"].createInstance(Components.interfaces.nsIDOMSerializer);
				var xmlStr = fc_gUtils.FC_XML_DEC + serializer.serializeToString(doc);
					
				// AFM - create stream - write, create, truncate
				//
				var outStream = Components.classes["@mozilla.org/network/file-output-stream;1"].
						createInstance(Components.interfaces.nsIFileOutputStream);
						
				outStream.init(fp.file, 0x02 | 0x08 | 0x20, 0664, 0);
				outStream.write(xmlStr, xmlStr.length);
				outStream.close();
				
				var promptText = document.getElementById("fc-dialog-export-success").getAttribute("label");
				FoxClocks_openSimpleInfo("FoxClocks", promptText);
			} 
			catch(ex) 
			{
				var promptText = document.getElementById("fc-dialog-export-failure").getAttribute("label") + ": " + ex;
				fc_gPromptService.alert(window, "FoxClocks", promptText);
			}
		}
		
		fc_gLogger.log("-FoxClocks::onExportCmd()");
	},
		
	// ====================================================================================
	onAddAsCmd : function()
	{		
		var location = this.getSelectedZonePickerLocation();
		if (!location)
			return;
		
		var styledLocationData = fc_gZoneManager.createStyledLocationData(location);
		var retLocation = this.openZoneInfo(styledLocationData, "LOCATION");			
			
		if (retLocation)
		{			
			styledLocationData.location = retLocation;
			
			var addedLocations = new Array();
			addedLocations.push(styledLocationData);
		
			this.addLocations(addedLocations);
			this.setWatchlistStates();
			this.setZonesPref();
			this.removeSortAttributes();
		}
	},
	
	// ====================================================================================
	onRemoveCmd : function()
	{
		fc_gLogger.log("+FoxClocks::onRemoveCmd()");
		
		var prompt = Components.classes["@mozilla.org/embedcomp/prompt-service;1"]
				.getService(Components.interfaces.nsIPromptService);
		var promptText = document.getElementById("fc-remove-confirm-label").getAttribute("label");  
		                        
		var result = prompt.confirm(window, null, promptText);
		
		if (result == false)
			return;
			
		var indexesToRemove = new Array();
											
		var rangeCount = this.watchList.view.selection.getRangeCount();
		for (var i=0; i < rangeCount; i++)
		{
			var start = {};
			var end = {};
			this.watchList.view.selection.getRangeAt(i, start, end);
			
			for (var c = start.value; c != -1 && c <= end.value; c++)
			{
				indexesToRemove.push(c);
			}
		}
			
		var treeChildrenRoot = document.getElementById("fc-watchlist-treechildren-root");

		// AFM - suppress select events on the watchlist until watchList and watchListData are
		// synchronised - we get called back in setWatchListStates() (the onselect handler)
		// immediately after we set the suppressed flag back to false
		//
		this.watchList.view.selection.selectEventsSuppressed = true;
		for (var i=indexesToRemove.length - 1; i >= 0; i--)
		{
			var indexToRemove = indexesToRemove[i];
			var item = this.watchList.view.getItemAtIndex(indexToRemove);
			item.parentNode.removeChild(item); 
			this.watchListData[indexToRemove] = null;
		}
		
		// AFM - sorry about this
		//
		var j = 0;
				
		while (j < this.watchListData.length)
		{	
			if (this.watchListData[j] == null)
				this.watchListData.splice(j, 1);
			else
				j++;
		}
		
		this.watchList.view.selection.selectEventsSuppressed = false;
					
		// this.setWatchlistStates(); - not necessary - see above
		this.setZonesPref();
		// this.removeSortAttributes(); - not necessary
		
		fc_gLogger.log("-FoxClocks::onRemoveCmd()");
	},
	
	// ====================================================================================
	onMoveUpCmd : function()
	{
		// AFM - we know only one item is selected and moving up is possible, saccess to the event is
		// disabled otherwise
		//
		var selectedItem = this.getSelectedWatchlistItem();
		var prevItem = selectedItem.previousSibling;
		this.swapWatchlistItems(prevItem, selectedItem);
		
		this.setZonesPref();
		this.removeSortAttributes();
		
		// AFM - need to update button states, even though the selection hasn't changed,
		// eg if we've moved an item to the top of the list
		//
		
		// setWatchlistStates();
		
		// AFM - actually, reselect the selected item - the selection is lost moving up. This will trigger
		// the above call.
		//
		this.watchList.view.selection.select(this.watchList.view.getIndexOfItem(selectedItem));
	},
	
	// ====================================================================================
	onMoveDownCmd : function()
	{	
		// AFM - we know only one item is selected and moving down is possible, access to the event is
		// disabled otherwise
		//
		var selectedItem = this.getSelectedWatchlistItem();
		var nextItem = selectedItem.nextSibling;
		this.swapWatchlistItems(selectedItem, nextItem);
		
		this.setZonesPref();
		this.removeSortAttributes();
		
		// AFM - need to update button states, even though the selection hasn't changed,
		// eg if we've moved an item to the bottom of the list
		//
		this.setWatchlistStates();
	},
	
	// ====================================================================================
	onWatchlistSelect : function() { this.setWatchlistStates(); },
	onOpenOptionsCmd : function() { FoxClocks_openWindow("chrome://foxclocks/content/options.xul", "", "chrome,centerscreen,resizable=no"); },
	onOpenAboutCmd : function() { FoxClocks_openWindow("chrome://foxclocks/content/about.xul", "", "chrome,dialog,modal,centerscreen,resizable=no"); },
	
	// ====================================================================================
	onOpenLocDetailsCmd : function()
	{
		var locationItem = this.getSelectedWatchlistItem();
		if (locationItem == null)
			return;
			
		var locationIndex = this.watchList.view.getIndexOfItem(locationItem);
		var styledLocationData = this.watchListData[locationIndex];
		
		var retLocation = this.openZoneInfo(styledLocationData, "LOCATION");
		
		if (retLocation)
		{
			var oldName = styledLocationData.location.name;
					
			styledLocationData.location = retLocation;
			var newName = styledLocationData.location.name;
			
			// AFM - shortcut - don't need to regenerate whole watchlist
			//
			locationItem.firstChild.firstChild.setAttribute("label", newName);
			
			this.setWatchlistStates();
			this.setZonesPref();
			
			if (oldName != newName)
				this.removeSortAttributes();
		}
	},
	
	// ====================================================================================
	onOpenLocGoogleEarthCmd : function()
	{
		var locationItem = this.getSelectedWatchlistItem();
		
		if (locationItem == null)
			return;
			
		var locationIndex = this.watchList.view.getIndexOfItem(locationItem);
		this.openGoogleEarth(this.watchListData[locationIndex].location);
	},
	
	// ====================================================================================
	onToggleBarClocksCmd : function()
	{
		fc_gLogger.log("+FoxClocks::onToggleBarClocksCmd()");
		
		var rangeCount = this.watchList.view.selection.getRangeCount();
		for (var i=0; i < rangeCount; i++)
		{
			var start = {};
			var end = {};
			this.watchList.view.selection.getRangeAt(i, start, end);
			
			for (var c = start.value; c != -1 && c <= end.value; c++)
			{
				var styledLocationData = this.watchListData[c];
				styledLocationData.showClock_statusbar = !styledLocationData.showClock_statusbar;
			}
		}
		
		this.setWatchlistStates();
		this.setZonesPref();
		
		fc_gLogger.log("+FoxClocks::onToggleBarClocksCmd()");
	},
	
	// ====================================================================================
	onTimer : function(scope)
	{
		// fc_gLogger.log("+FoxClocks::onTimer()");
					
		if (scope == null)
			scope = this;
		
		var nowDate = new Date();		
		for (var j=0; j < scope.watchList.view.rowCount; j++)
		{			
			var treeItem = scope.watchList.view.getItemAtIndex(j);
			
			// AFM - historical - keeping around
			//
			if (treeItem.getAttribute("id") == "fc-watchlist-noitems-item")
				continue;
				
			var location = scope.watchListData[j].location;
			var timeText = scope.timeFormatter.getTimeString(location, nowDate);
			
			var timeCell = treeItem.firstChild.childNodes[1];
			timeCell.setAttribute("label", timeText);
			
			// AFM - doesn't work since Watchlist changed from listbox to tree
			//
			var tooltipText = "GMT/UTC" + location.zone.currOffsetString(nowDate) +
					" (" + location.zone.currName(nowDate) + ")";
							
			if (location.zone.isDST(nowDate))
				tooltipText += "*";
				
			timeCell.setAttribute("tooltiptext", tooltipText);
		}
		
		// fc_gLogger.log("-FoxClocks::onTimer()");
	},
	
	// ====================================================================================
	// AFM - implementing nsIObserver
	//	
	observe : function(subject, topic, data)
	{
		if (topic == "foxclocks")
		{
			if (data == "engine:zone-data-update-complete")
			{
				if (subject.wrappedJSObject.lastUpdateResult == "OK_NEW")
				{
					this.populateZonePicker();
					this.populateWatchlist();
				}
			}
			else if (data == "engine:zone-picker-changed")
			{
				this.populateZonePicker();
			}
		}
		else if (topic == "nsPref:changed")
		{
			switch (data)
			{
				case "foxclocks.format.foxclocks.standard": this.setTimeFormat(); break;
				case "foxclocks.format.foxclocks.custom": this.setTimeFormat(); break;
				case "foxclocks.clock.style": this.setWatchlistStates(); break;
				case "foxclocks.zones": this.populateWatchlist(); break;
			}
		}
		
		this.onTimer();
	},
	
	//
	// AFM - END, EVENTS
	
	// ====================================================================================
	populateWatchlist : function()
	{
		// AFM - completely regenerate watchlist, when driven by external pref changes
		// (eg importing settings), or on load
		//
		
		var treeChildrenRoot = document.getElementById("fc-watchlist-treechildren-root");
		
		while(treeChildrenRoot.hasChildNodes())
			treeChildrenRoot.removeChild(treeChildrenRoot.firstChild);
  	
		var addedLocations = new Array();
	
		var zonesParam = fc_gPrefManager.getPref("foxclocks.zones").split(fc_gUtils.FC_LOCATION_DELIM);
	
		for (var i=0; zonesParam != "" && i < zonesParam.length; i++)
		{		
			var styledLocationData = fc_gZoneManager.createStyledLocationData(null);
			if (styledLocationData.populateFromZoneParam(zonesParam[i]) == false)
				continue;
			
			addedLocations.push(styledLocationData);
		}
		
		this.watchListData = new Array(); // ie blow it away
		this.addLocations(addedLocations);
		this.setWatchlistStates();
	},
	
	// ====================================================================================
	populateZonePicker : function()
	{
		fc_gLogger.log("+FoxClocks::populateZonePicker()");

		var treeChildrenRoot = document.getElementById("fc-zonepicker-treechildren-root");
		
		while(treeChildrenRoot.hasChildNodes())
			treeChildrenRoot.removeChild(treeChildrenRoot.firstChild);
			
		var nowDate = new Date();
		var zonePickerArray = fc_gZoneManager.getZonePickerArray();
		
		for (var i=0; i < zonePickerArray.length; i++)
		{
			var currLocation = zonePickerArray[i]; // Yes, there are locations in the array, not zones
			var zoneNameAsArray = currLocation.zp_structure.split('|');
			
			for (var k=0; k < zoneNameAsArray.length; k++)
			{
				var parentTreeChildren = k == 0 ? treeChildrenRoot : document.getElementById(this.getTreeChildrenId(k - 1, zoneNameAsArray));
					
				if (k < zoneNameAsArray.length - 1) // i.e. is not terminal node
				{		
					var childTreeChildren = document.getElementById(this.getTreeChildrenId(k, zoneNameAsArray));
					if (childTreeChildren == null)
					{
						childTreeChildren = document.createElement("treechildren");
						childTreeChildren.setAttribute("id", this.getTreeChildrenId(k, zoneNameAsArray));
						
						var treeItem1 = document.createElement("treeitem");
						treeItem1.setAttribute("container", "true");
						
						parentTreeChildren.appendChild(treeItem1);
						
						var treeCell1 = document.createElement("treecell");
						treeCell1.setAttribute("label", zoneNameAsArray[k]);
						
						var treeRow1 = document.createElement("treerow");
						
						treeRow1.appendChild(treeCell1);
						treeItem1.appendChild(treeRow1);
						treeItem1.appendChild(childTreeChildren);
					}
				}
				else
				{										
					var treeItem2 = document.createElement("treeitem");
					treeItem2.setAttribute("value", i); // AFM - to allow lookup in this.zonePickerArray
					
					parentTreeChildren.appendChild(treeItem2);
					
					var cellCurrZoneLabel = currLocation.zone.currOffsetString(nowDate) +
						" (" + currLocation.zone.currName(nowDate) + ")";
		
					if (currLocation.zone.isDST(nowDate))
						cellCurrZoneLabel += "*";
					
					var treeCellRegion = document.createElement("treecell");
					treeCellRegion.setAttribute("label", zoneNameAsArray[k]);
		
					var treeCellCurrZone = document.createElement("treecell");
					treeCellCurrZone.setAttribute("label", cellCurrZoneLabel);
		
					var treeRow2 = document.createElement("treerow");					
					treeRow2.appendChild(treeCellRegion);
					treeRow2.appendChild(treeCellCurrZone);
					
					treeItem2.appendChild(treeRow2);
				}
			}
		}
		
		this.setZonePickerStates();
		
		fc_gLogger.log("-FoxClocks::populateZonePicker()");
	},
	
	// ====================================================================================
	addLocations : function(addedLocations)
	{
		fc_gLogger.log("+FoxClocks::addLocations()");
			
		var treeChildrenRoot = document.getElementById("fc-watchlist-treechildren-root");
		
		for (var i=0; addedLocations != "" && i < addedLocations.length; i++)
		{
			var styledLocationData = addedLocations[i];
			var location = styledLocationData.location;
			
			var locationCell = document.createElement("treecell");
			locationCell.setAttribute("label", location.name);
															
			var treeRow = document.createElement("treerow");
			treeRow.appendChild(locationCell);
			treeRow.appendChild(document.createElement("treecell"));
					
			var treeItem = document.createElement("treeitem");
			treeItem.setAttribute("value", location.zone.id);
			treeItem.appendChild(treeRow);

			treeChildrenRoot.appendChild(treeItem);										
			this.watchListData.push(styledLocationData);
		}
		
		fc_gLogger.log("-FoxClocks::addLocations()");
	},

	// ====================================================================================
	sortWatchlist : function(event)
	{
		// AFM - pretty ugly and inefficient, but it works
		// if event is null, we're being called to re-sort on given current resource (column)
		// (not currently used)
		
		fc_gLogger.log("+FoxClocks::sortWatchlist()");
	
		// AFM - nothing to do
		//
		var sortResourse = this.watchList.getAttribute("sortResource");
		if (event == null && sortResource == "")
			return;
			
		var column = event != null ? event.target : document.getElementById(sortResource);
		var colId = column.getAttribute("id");
		var colSortDir = column.getAttribute("sortDirection");
		
		fc_gLogger.log("FoxClocks::sortWatchlist(): " + colId + ", sort dir <" + colSortDir + ">");		
		
		var newSortDir = colSortDir == "ascending" || colSortDir == "descending" ?
							colSortDir : "descending";
		
		// AFM toggle sort order if we're not re-sorting on current resource
		//
		if (event != null)
			newSortDir = newSortDir == "descending" ? "ascending" : "descending";
		
		var currDate = new Date();
									
		function columnSort(aa, bb)
		{	
			function nameSort(a, b) {
				return	a.location.name > b.location.name ? 1 :
						(a.location.name == b.location.name ? 0 : -1); }
					
			function timeSort(a, b) {
				return	a.location.zone.currDisplayDate(currDate) -
						b.location.zone.currDisplayDate(currDate); }
					
			var a = newSortDir == "ascending" ? aa : bb;
			var b = newSortDir == "ascending" ? bb : aa;
			
			var primarySort = colId == "fc_watchlist_col_location" ? nameSort : timeSort;
			var secondarySort = colId == "fc_watchlist_col_location" ? timeSort : nameSort;
			
			var retVal = primarySort(a, b);
			if (retVal == 0)
				retVal = secondarySort(aa, bb); // always ascending
				
			return retVal;
		}	
		
		var cols = this.watchList.getElementsByTagName("treecol");
		for (var i = 0; i < cols.length; i++)
		{
			cols[i].removeAttribute("sortDirection");
		}
		
		column.setAttribute("sortDirection", newSortDir);
		this.watchList.setAttribute("sortResource", colId);
		
		this.watchListData.sort(columnSort);
		this.setZonesPref(); // since we've updated watchListData
		
		fc_gLogger.log("-FoxClocks::sortWatchlist(): intermed");
		
		// AFM - we don't observe pref changes while in this.setPrefs()
		// populateWatchlist() calls setWatchlistStates()
		//
		this.populateWatchlist();
		
		fc_gLogger.log("-FoxClocks::sortWatchlist(): " + newSortDir);
	},		
	
	// ====================================================================================
	removeSortAttributes : function()
	{
		// AFM - we don't re-sort when a location is added or re-named etc, so we should
		// remove the sort arrows when we do something like this. Aside: we also don't re-sort
		// when a zone changes time zone... Pretty minor
		//
		var cols = this.watchList.getElementsByTagName("treecol");
		for (var i = 0; i < cols.length; i++)
		{
			cols[i].removeAttribute("sortDirection");
		}
		
		this.watchList.removeAttribute("sortResource");
	},
	
	// ====================================================================================
	setTimerInterval : function()
	{
		if (this.hTimer != null)
			window.clearInterval(this.hTimer);
			
		this.hTimer = window.setInterval(this.onTimer, fc_gUtils.FC_CLOCK_UPDATE_INTERVAL, this);
	},
	
	// ====================================================================================
	setTimeFormat : function()
	{
		var standardFormat = fc_gPrefManager.getPref("foxclocks.format.foxclocks.standard");
		var customFormat = fc_gPrefManager.getPref("foxclocks.format.foxclocks.custom");
										
		this.timeFormatter.setTimeFormat(standardFormat == "" ? customFormat : standardFormat);
	},
	
	// ====================================================================================
	setZonesPref : function()
	{
		fc_gLogger.log("+FoxClocks::setZonesPref(): pref is <" + fc_gPrefManager.getPref("foxclocks.zones") + ">");
	
		var zonePrefsString = "";
		var isFirstIndex = true;
		
		for (var i=0; i < this.watchListData.length; i++)
		{
			var styledLocationData = this.watchListData[i];
			
			if (isFirstIndex)
				isFirstIndex = false;
			else
				zonePrefsString += fc_gUtils.FC_LOCATION_DELIM;
			
			zonePrefsString += styledLocationData.toZoneParam();
		}
		
		// AFM - we don't want to respond to our own pref change - specifically, we don't 
		// want to trigger this.populateWatchlist()
		//
		fc_gPrefManager.removePrefObserver("foxclocks.", this);
		fc_gPrefManager.setPref("foxclocks.zones", zonePrefsString);
		fc_gPrefManager.addPrefObserver("foxclocks.", this);
		
		fc_gLogger.log("-FoxClocks::setZonesPref(): pref is <" + fc_gPrefManager.getPref("foxclocks.zones") + ">");
	},
	
	// ====================================================================================
	getTreeChildrenId : function(index, array) { return "fc-zonepicker-treechildren:" + array.slice(0, index + 1).join(""); },
	
	// ====================================================================================
	getSelectedZonePickerLocation: function()
	{
		if (this.zonePicker.view.selection.count != 1)
			return null;
			
		var start = {};
		var end = {};
		this.zonePicker.view.selection.getRangeAt(0, start, end);

		var zoneItem = this.zonePicker.view.getItemAtIndex(start.value);
		
		if (zoneItem.getAttribute("container") == "true")
			return null;

		return fc_gZoneManager.getZonePickerArray()[zoneItem.getAttribute("value")];
	},
	
	// ====================================================================================
	getSelectedWatchlistItem : function()
	{
		if (this.watchList.view.selection.count == 1)
		{
			var start = {};
			var end = {};
			this.watchList.view.selection.getRangeAt(0, start, end);

			return this.watchList.view.getItemAtIndex(start.value);
		}
		
		return null;
	},
	
	// ====================================================================================
	openZoneInfo : function(styledLocationData, mode)
	{
		var retVals = {location: null};
		
		window.openDialog(	"chrome://foxclocks/content/zoneinfo.xul", "",
							"chrome,modal,centerscreen,resizable=no",
							this.timeFormatter, mode, styledLocationData, retVals);
							
		return retVals.location;
	},
	
	// ====================================================================================
	openGoogleEarth : function(location)
	{					
		try
		{
			fc_gUtils.openGoogleEarth(location);
		}
		catch (ex)
		{
			var errorMsg = document.getElementById("foxclocks-bundle").getString("misc.msg.googleeartherror");
			fc_gPromptService.alert(window, "FoxClocks", errorMsg + ": " + ex);
		}
	},

	// ====================================================================================
	setZonePickerStates : function()
	{
		// AFM - enable/disable things that depend on what's selected in the zone picker
		//
		
		fc_gLogger.log("+FoxClocks::setZonePickerStates()");
		
		var location = this.getSelectedZonePickerLocation();
		
		FoxClocks_DisableId("cmd_fc_add", this.zonePicker.view.selection.getRangeCount() == 0);
		FoxClocks_DisableId("cmd_fc_addas", location == null);
		FoxClocks_DisableId("cmd_fc_zonedetails", location == null);
		
		FoxClocks_DisableId("cmd_fc_zonegoogleearth",
				location == null || location.getLatitude() == null || location.getLongitude() == null);
		
		fc_gLogger.log("-FoxClocks::setZonePickerStates()");
	},
	
	// ====================================================================================
	setWatchlistStates : function()
	{
		fc_gLogger.log("+FoxClocks::setWatchlistStates()");
	
		/* AFM - commenting out '(no locations selected)' code - can't disable any of
		   the treexxx widgets, and the message only appears in one column anyway, so
		   can be truncated
		   
		var noItemsItem = document.getElementById("fc-watchlist-noitems-item");
		var treeChildrenRoot = document.getElementById("fc-watchlist-treechildren-root");
					
		if (this.watchListData.length == 0 && noItemsItem == null)
		{	
			var label = document.getElementById("fc-watchlist-nolocs-label").getAttribute("label");
			var locationCell = document.createElement("treecell");
			locationCell.setAttribute("label", label);
			locationCell.setAttribute("disabled", "true");
															
			var treeRow = document.createElement("treerow");
			treeRow.setAttribute("disabled", "true");
			treeRow.appendChild(locationCell);
			
			var treeItem = document.createElement("treeitem");
			treeItem.setAttribute("id", "fc-watchlist-noitems-item");
			treeItem.setAttribute("disabled", "true");
			treeItem.appendChild(treeRow);
			
			treeChildrenRoot.appendChild(treeItem);
		}
		else if (this.watchListData.length > 0 && noItemsItem != null)
		{
			treeChildrenRoot.removeChild(noItemsItem);
		}			
		*/
					
		// AFM - check everything that should be checked
		// Bit of a ratty non-optimal piece of code...
		//
		var selectedCount = this.watchList.view.selection.count;
		var selectedItem = this.getSelectedWatchlistItem();
		var selectedIndex = selectedItem ? this.watchList.view.getIndexOfItem(selectedItem) : null;
		var prevItem = selectedItem ? selectedItem.previousSibling : null;
		var nextItem = selectedItem ? selectedItem.nextSibling : null;
			
		FoxClocks_DisableId("cmd_fc_moveup", (prevItem == null));
		FoxClocks_DisableId("cmd_fc_movedown", (nextItem == null));
		FoxClocks_DisableId("cmd_fc_remove", (selectedCount == 0));	
		FoxClocks_DisableId("cmd_fc_locdetails", (selectedItem == null));
		FoxClocks_DisableId("cmd_fc_toggle_bar_clocks", (selectedCount == 0));
		
		var location = selectedItem ? this.watchListData[selectedIndex].location : null;
		FoxClocks_DisableId("cmd_fc_locgoogleearth",
				location == null || location.getLatitude() == null || location.getLongitude() == null);
		
		this.onTimer();
		
		fc_gLogger.log("-FoxClocks::setWatchlistStates()");
	},		
	
	// ====================================================================================
	swapWatchlistItems : function(firstItem, secondItem)
	{
		// AFM - swap items in watchListData
		//
		var firstLocationIndex = this.watchList.view.getIndexOfItem(firstItem);
		var secondLocationIndex = this.watchList.view.getIndexOfItem(secondItem);

		fc_gLogger.log("FoxClocks::swapWatchlistItems(): " + firstLocationIndex + " " + secondLocationIndex);
		
		var firstStyledLocationData = this.watchListData[firstLocationIndex];
		var secondStyledLocationData = this.watchListData[secondLocationIndex];
		
		this.watchListData[firstLocationIndex] = secondStyledLocationData;
		this.watchListData[secondLocationIndex] = firstStyledLocationData;
		
		// AFM - swap items in watchList
		//
		document.getElementById("fc-watchlist-treechildren-root").insertBefore(secondItem, firstItem);
	}
}