// FoxClocks extension for Mozilla Firefox/Thunderbird/Sunbird
// Copyright (C) 2006 Andy McDonald / www.stemhaus.com
// For licensing terms, please refer to readme.txt in this extension's '.xpi'
// package or its installation directory on your computer.
	
// ====================================================================================
function FoxClocks_Overlay()
{			
	this.clockTimeFormatter = null;
	this.tooltipTimeFormatter = null;
	this.hTimer = null;
	
	// AFM - lots of duplication of this.watchListData in foxclocks.js - to replace with component
	// Here location array is an array of FoxClocks_Pairs of StyledLocationData and clocks
	//
	this.locationArray = new Array();
	this.clockTooltip = null;
	this.clockTooltipUpdate = false;
	this.clockOldContainer = null;
	this.clockContainerType = null;
	this.hCustomizeToolbarWindow = null;
	
	this.observeRecursionDepth = 0;
}
	
// ====================================================================================
FoxClocks_Overlay.prototype =
{
	// ====================================================================================
	onLoad : function()
	{
		fc_gLogger.log("+FoxClocks_Overlay::onLoad()");
		
		this.clockTimeFormatter = new FoxClocks_TimeFormatter();
		this.tooltipTimeFormatter = new FoxClocks_TimeFormatter();
	
		this.clockTooltip = document.getElementById("foxclocksoverlay-clock-tooltip");
		 
		this.setClocksContainerType();
		this.setClocksFormat();
		this.setTooltipFormat();
		this.setTimerInterval();
		
		this.createClocks();
		this.updateView();
		
		fc_gPrefManager.addPrefObserver("foxclocks.", this);
				
		fc_gObserverService.addObserver(this, "domwindowopened", false);
		fc_gObserverService.addObserver(this, "domwindowclosed", false);
		fc_gObserverService.addObserver(this, "foxclocks", false);
		
		// AFM - use timeout to avoid app not being fully rendered when welcome/update window opens
		// Should move to engine
		//
		setTimeout(this.checkVersion, 0, this);
		
		fc_gLogger.log("-FoxClocks_Overlay::onLoad()");
	},
		
	// ====================================================================================	
	onUnload : function()
	{
		if (this.hTimer != null)
			window.clearInterval(this.hTimer);
			
		fc_gPrefManager.removePrefObserver("foxclocks.", this);
			
		fc_gObserverService.removeObserver(this, "domwindowopened");
		fc_gObserverService.removeObserver(this, "domwindowclosed");
		fc_gObserverService.removeObserver(this, "foxclocks");
	},
	
	// ====================================================================================
	onToolbarButtonCommand : function(event) { this.openFoxClocksWindow(); },
	onClockIconClick : function(event) { if (event.button == 0) this.openFoxClocksWindow(); },
	onClockDblClick : function(event) { if (event.button == 0) this.openFoxClocksWindow(); },
	onClockTooltipShowing : function(event) { this.clockTooltipUpdate = true; this.updateView(); },
	onClockTooltipHidden : function(event) { this.clockTooltipUpdate = false; },
	
	// ====================================================================================
	// AFM - implementing nsIObserver
	//
	observe : function(subject, topic, data, self)
	{
		var recursing = false;
		
		// AFM - potentially in this method from setTimeout() callback
		//
		if (self == null)
			self = this;
	
		// fc_gLogger.log("FoxClocks_Overlay::observe(): topic: <" + topic + ">, clock location <" +
		//	self.clockContainerType + ">");
		
		if (topic == "foxclocks")
		{
			if (data == "engine:zone-data-update-complete")
			{		
				if (subject.wrappedJSObject.lastUpdateResult == "OK_NEW")
				{
					self.createClocks();
				}
			}			
		}
		else if (topic == "nsPref:changed")
		{
			switch (data) // prefName
			{
				case "foxclocks.zones": self.createClocks(); break;
				case "foxclocks.clock.containertype": self.setClocksContainerType(); self.createClocks(); break;
				case "foxclocks.clock.style": self.setClocksVisibility(); break;
				case "foxclocks.clock.position.relative": self.createClocks(); break;
				case "foxclocks.format.clock.standard": self.setClocksFormat(); break;
				case "foxclocks.format.clock.custom": self.setClocksFormat(); break;
				case "foxclocks.format.tooltip.standard": self.setTooltipFormat(); break;
				case "foxclocks.format.tooltip.custom": self.setTooltipFormat(); break;
			}
		}
		else if (topic == "domwindowclosed")
		{
			if (subject.location != null && subject.location.href == FoxClocks_getCustomizeToolbarURL())
			{
				if (self.hCustomizeToolbarWindow == null)
					fc_gLogger.log("FoxClocks_Overlay::observe(): customize toolbar window closed, but open event not received", fc_gLogger.WARN);
				else
					fc_gLogger.log("FoxClocks_Overlay::observe(): customize toolbar window closed");
					
				self.hCustomizeToolbarWindow = null;
				
				// AFM - regenerate dynamic content of toolbar when the customize dialog closed
				// Actually do this whether or not we're in toolbar mode
				//
				self.createClocks();
				self.updateView();
			}
		}
		else if (topic == "domwindowopened")
		{		
			if (subject.location != null && subject.location.href == "about:blank")
			{
				const MAX_RECURSE_DEPTH = 2;
				if (self.observeRecursionDepth < MAX_RECURSE_DEPTH)
				{
					fc_gLogger.log("FoxClocks_Overlay::observe(): unknown window opened - recursing (depth curr. "
						+ self.observeRecursionDepth + ")");
					
					// AFM - try to get the href again. Only recurse MAX_RECURSE_DEPTH times, just in case
					//
					self.observeRecursionDepth++;
					recursing = true;
					setTimeout(self.observe, 0, subject, topic, data, self);
				}
				else
				{
					// AFM - log at debug - seems to happen a fair bit, and we don't want to worry people
					//
					fc_gLogger.log("FoxClocks_Overlay::observe(): reached max recursion depth "
						+ MAX_RECURSE_DEPTH);
				}
				
			}
			else if (subject.location != null && subject.location.href == FoxClocks_getCustomizeToolbarURL())
			{
				fc_gLogger.log("FoxClocks_Overlay::observe(): customize toolbar window opened");
				self.hCustomizeToolbarWindow = subject;

				self.createClocks();
				self.updateView();
			}
			else
			{
				var href = subject.location != null ? subject.location.href : "- subject.location null -";
				fc_gLogger.log("FoxClocks_Overlay::observe(): <" + href + "> opened");
			}
		}
		
		if (recursing == false)
			self.observeRecursionDepth = 0;
			
		self.updateView();
	},
	
	// ====================================================================================
	checkVersion : function(self)
	{			
		var prevRunVersion = fc_gPrefManager.getPref("extensions." +
					fc_gUtils.FC_GUID_FOXCLOCKS + ".prevrun.version");
					
		var currentVersion = fc_gUtils.getFoxClocksVersion();
					
		if (currentVersion)
			fc_gPrefManager.setPref("extensions." + fc_gUtils.FC_GUID_FOXCLOCKS + ".prevrun.version", currentVersion);
		
		// AFM - don't display enable automatic updates checkbox if it's currently enabled
		//
		var hideEnableUpdate = fc_gPrefManager.getPref("foxclocks.data.update.auto.enabled");

		if (prevRunVersion == "") // first run
		{
			var retVals = {openFoxClocks: false, launchHomePage: false, enableAutoUpdate: false};
			window.openDialog(	"chrome://foxclocks/content/welcome.xul", "",
								"chrome,modal,centerscreen,resizable=no", retVals, hideEnableUpdate);

			if (retVals.launchHomePage)
				FoxClocks_openFoxClocksHomeURL(false); // do not pass in progress listener - forget it
																		
			if (retVals.openFoxClocks)
				self.openFoxClocksWindow();

			if (hideEnableUpdate == false)
			{
				// AFM - should be going through a component, not hitting the
				// pref directly
				//
				fc_gPrefManager.setPref("foxclocks.data.update.auto.enabled", retVals.enableAutoUpdate);
				fc_gLogger.log("FoxClocks_Overlay::checkVersion(): auto-update on welcome: " + retVals.enableAutoUpdate);
			}
			else
			{
				// AFM - wouldn't expect to get here on welcome, but theoretically someone
				// could be messing with the prevrun.version param
				//
				fc_gLogger.log("FoxClocks_Overlay::checkVersion(): auto-update on welcome hidden");
			}
			return;
		}
		
		var comparison = Components.classes["@mozilla.org/xpcom/version-comparator;1"]
			.getService(Components.interfaces.nsIVersionComparator).compare(prevRunVersion, currentVersion);
			
		if (comparison < 0)
		{				
			var retVals = {launchChangeLog: false, enableAutoUpdate: false};
			window.openDialog(	"chrome://foxclocks/content/update.xul", "",
								"chrome,modal,centerscreen,resizable=no", retVals, hideEnableUpdate);
								
			if (retVals.launchChangeLog)
				FoxClocks_openFoxClocksHomeURL(true);
				
			if (hideEnableUpdate == false)
			{
				// AFM - should be going through a component, not hitting the
				// pref directly
				//
				fc_gPrefManager.setPref("foxclocks.data.update.auto.enabled", retVals.enableAutoUpdate);
				fc_gLogger.log("FoxClocks_Overlay::checkVersion(): auto-update on upgrade: " + retVals.enableAutoUpdate);
			}
			else
			{
				fc_gLogger.log("FoxClocks_Overlay::checkVersion(): auto-update on upgrade hidden");
			}
		}
		else if (comparison > 0)
		{
			fc_gLogger.log("FoxClocks_Overlay::checkVersion(): FoxClocks downgraded from <" +
				prevRunVersion + "> to <" + currentVersion + ">", fc_gLogger.WARN);
		}
	},
	
	// ====================================================================================
	updateView : function(scope)
	{	
		if (scope == null)
			scope = this;
		
		var nowDate = new Date();
		var clockAttType = scope.getClockXULAttributeType();
		
		// AFM - update clocks if visible
		//
		for (var i=0; i < scope.locationArray.length; i++)
		{
			var styledLocationData = scope.locationArray[i].first;
			var clock = scope.locationArray[i].second;
			
			if (!clock.hidden)
			{
				clock.setAttribute(clockAttType,
					scope.clockTimeFormatter.getTimeString(styledLocationData.location, nowDate));
				
				var colour = styledLocationData.colour != "" ? styledLocationData.colour : "-moz-dialogtext";
				
				if (styledLocationData.altColour_enabled)
				{
					var localDate = styledLocationData.location.zone.currDisplayDate(nowDate);
					var minsPastMidnight = localDate.getUTCHours() * 60 + localDate.getUTCMinutes();
					
					var startTime = styledLocationData.altColour_startTime;
					var endTime = styledLocationData.altColour_endTime;
					
					if (startTime < endTime)
					{
						if (minsPastMidnight >= startTime && minsPastMidnight < endTime)
							colour = styledLocationData.altColour;
					}
					else
					{
						if (!(minsPastMidnight >= endTime && minsPastMidnight < startTime))
							colour = styledLocationData.altColour;
					}			
					
					// fc_gLogger.log(styledLocationData.location.name + " " + startTime + " " + minsPastMidnight + " " + endTime);
				}
					
				clock.setAttribute("style", "color:" + colour + ";");
			}
		}
	
		// AFM - update tooltip if visible
		//
		var clockTooltipGrid = scope.clockTooltip.firstChild;
		if (clockTooltipGrid && scope.clockTooltipUpdate)
		{
			var gridRows = clockTooltipGrid.lastChild;
			for (var j=0; j < gridRows.childNodes.length; j++)
			{
				var gridRow = gridRows.childNodes[j];
				var locationIndex = gridRow.getAttribute("value");
				
				if (locationIndex && locationIndex != "fc-nolocs-row")
				{
					var location = scope.locationArray[locationIndex].first.location;
					var timeText = scope.tooltipTimeFormatter.getTimeString(location, nowDate);	
					gridRow.lastChild.setAttribute("value", timeText);
				}
			}
		}
	},
	
	// ====================================================================================
	setClocksContainerType : function()
	{
		this.clockContainerType = fc_gPrefManager.getPref("foxclocks.clock.containertype");
	},
	
	// ====================================================================================
	setClocksFormat : function()
	{
		var standardFormat = fc_gPrefManager.getPref("foxclocks.format.clock.standard");
		var customFormat = fc_gPrefManager.getPref("foxclocks.format.clock.custom");
										
		this.clockTimeFormatter.setTimeFormat(standardFormat == "" ? customFormat : standardFormat);
	},
	
	// ====================================================================================
	setTooltipFormat : function()
	{
		var standardFormat = fc_gPrefManager.getPref("foxclocks.format.tooltip.standard");
		var customFormat = fc_gPrefManager.getPref("foxclocks.format.tooltip.custom");
										
		this.tooltipTimeFormatter.setTimeFormat(standardFormat == "" ? customFormat : standardFormat);
	},
	
	// ====================================================================================
	setTimerInterval : function()
	{
		if (this.hTimer != null)
			window.clearInterval(this.hTimer);
						
		this.hTimer = window.setInterval(this.updateView, fc_gUtils.FC_CLOCK_UPDATE_INTERVAL, this);
	},
	
	// ====================================================================================
	setClocksVisibility : function()
	{
		var clockStyle = fc_gPrefManager.getPref("foxclocks.clock.style");
		
		var showToolbarButton = (this.clockContainerType == "fc-clock-containertype-toolbar" && (clockStyle == "fc-clock-style-icon" || this.hCustomizeToolbarWindow));
		var showStatusbarIcon = (this.clockContainerType == "fc-clock-containertype-statusbar" && clockStyle == "fc-clock-style-icon");
		
		var hideAllClocks = (showToolbarButton || showStatusbarIcon);
		
		this.showClocksToolbarButton(showToolbarButton);
		
		var statusbarPanelIcon = document.getElementById("foxclocksoverlay-statusbarpanel-icon");
		
		if (statusbarPanelIcon)
			statusbarPanelIcon.setAttribute("hidden", (showStatusbarIcon ? "false" : "true"));
		
		for (var i=0; i < this.locationArray.length; i++)
		{
			var showClock = this.locationArray[i].first.showClock_statusbar;
			var clock = this.locationArray[i].second;
			
			clock.hidden = hideAllClocks || showClock == "0";
		}
	},

	// ====================================================================================
	createClocks : function()
	{
		fc_gLogger.log("+FoxClocks_Overlay::createClocks()");
				
		// AFM - subtle. Blow away the clocks and clockTooltipGrid
		//
		
		var statusbarPanelIcon = document.getElementById("foxclocksoverlay-statusbarpanel-icon");

		if (statusbarPanelIcon && this.clockOldContainer)
			this.clockOldContainer.removeChild(statusbarPanelIcon);
			
		var arrayLength = this.locationArray.length;
		for (var j=0; j < arrayLength; j++)
		{
			// AFM - number of child clocks may not sync with the locationArray e.g. when the
			// item's been dragged from the toolbar palette,
			// in which case there are no child clocks
			//
			this.locationArray.pop();
			var clock = document.getElementById("foxclocksoverlay-clock-id-" + j);
			
			// fc_gLogger.log("FoxClocks_Overlay::createClocks(): container <" + 
			//	(this.clockOldContainer ? this.clockOldContainer.getAttribute("id") : "null") + 
			//	"> - child with id <" + (clock ? clock.getAttribute("id") : "null") + ">");

			if (clock && this.clockOldContainer)
				this.clockOldContainer.removeChild(clock);
		}
		
		var clockTooltipGrid = this.clockTooltip.firstChild;
		
		if (clockTooltipGrid != null)
			this.clockTooltip.removeChild(clockTooltipGrid);

		var clockContainer = this.getClockContainer();
		var clockType = this.getClockXULType();
		
		this.clockOldContainer = clockContainer;
		
		if (!clockContainer)
		{
			// AFM - container dragged off toolbar, for example
			//
			this.setClocksVisibility();
			fc_gLogger.log("FoxClocks_Overlay::createClocks(): no container");
			return;
		}

		var clocksPosnRelative = fc_gPrefManager.getPref("foxclocks.clock.position.relative");
			
		// AFM - create tool tip grid/columns/column/rows elements
		//
		clockTooltipGrid = document.createElement("grid");
		clockTooltipGrid.setAttribute("flex", "1");
		this.clockTooltip.appendChild(clockTooltipGrid);
	
		var gridColumns = document.createElement("columns");
		gridColumns.appendChild(document.createElement("column"));
		gridColumns.appendChild(document.createElement("column"));
		clockTooltipGrid.appendChild(gridColumns);
		
		var gridRows = document.createElement("rows");
		clockTooltipGrid.appendChild(gridRows);
		
		var previousClock = null;
		
		var startIndex = this.clockContainerType == "fc-clock-containertype-statusbar" ? -1 : 0;
		var clockId = 0;
		
		var zonesParam = fc_gPrefManager.getPref("foxclocks.zones").split(fc_gUtils.FC_LOCATION_DELIM);
		if (zonesParam.length == 1 && zonesParam[0] == "")
			zonesParam = [];
			
		for (var i = startIndex; i < zonesParam.length; i++)
		{			
			var clock = null;
			
			if (i == -1)
			{
				clock = document.createElement(clockType);
				clock.setAttribute("id", "foxclocksoverlay-statusbarpanel-icon");
				clock.setAttribute("class", "statusbarpanel-iconic");
				clock.setAttribute("src", "chrome://foxclocks/skin/toolbarbutton_small.png");
				clock.setAttribute("onclick", "foxClocksOverlay.onClockIconClick(event)");
			}
			else
			{
				// AFM - validate zone param and generate the location
				// Again, horrendous dupe from foxclocks.js - need component
				//
				var styledLocationData = fc_gZoneManager.createStyledLocationData(null);
				if (styledLocationData.populateFromZoneParam(zonesParam[i]) == false)
					continue;
				
				// AFM - don't use i, in case we've 'continued' above
				//
				var clockIdString = "foxclocksoverlay-clock-id-" + clockId++;
				
				clock = document.createElement(clockType);
				clock.setAttribute("id", clockIdString);
				clock.setAttribute("ondblclick", "foxClocksOverlay.onClockDblClick(event)");
				
				// AFM - apply workaround to clocks to get alignment right in toolbars,
				// unless we're in Firefox 1.5+ menubar. The parent's parent thing is to handle
				// bizarre "wrapper-" id's - don't understand what's happening - just making
				// a best guess based on DOM Inspector
				//
				var applyStyleWorkaround = true;
				
				if (fc_gUtils.isFirefox15Up && this.clockContainerType == "fc-clock-containertype-toolbar")
				{				
					var toolbarItem = document.getElementById("foxclocksoverlay-toolbaritem-clocks");
					var toolbarItemParent = toolbarItem && toolbarItem.parentNode;
					var toolbarItemGrandParent = toolbarItemParent && toolbarItemParent.parentNode;
					
					if (toolbarItemParent && toolbarItemGrandParent) // AFM - should always exist
					{
						var parentId = toolbarItemParent.getAttribute("id");
						var grandParentId = toolbarItemGrandParent.getAttribute("id");
						
						if (	parentId && grandParentId &&
								(parentId == "toolbar-menubar" ||
								grandParentId == "toolbar-menubar"))
						{
							applyStyleWorkaround = false;
						}
					}
				}
				
				var classString = "";
				
				if (styledLocationData.bold == true) classString += "foxclocks-bold ";
				if (styledLocationData.italic == true) classString += "foxclocks-italic ";
				if (styledLocationData.underline == true) classString += "foxclocks-underline ";
				if (applyStyleWorkaround) classString += "foxclocks-alignment-workaround ";
				
				if (classString != "")
					clock.setAttribute("class", classString);
					
				this.locationArray.push(new FoxClocks_Pair(styledLocationData, clock));	

				// AFM - generate the tool tip grid row
				//
				if (styledLocationData.showClock_statusbarTooltip == true)
				{
					var gridRow = document.createElement("row");
					
					// AFM - store index of location corresponding to tooltip row
					// so we can get the location's time when updating the tooltip
					//
					gridRow.setAttribute("value", this.locationArray.length - 1);
					
					var label = document.createElement("label");
					label.setAttribute("value", styledLocationData.location.name);
				
					gridRow.appendChild(label);
					gridRow.appendChild(document.createElement("label"));
	
					gridRows.appendChild(gridRow);
				}
			}
			
			// AFM - attributes common to the 'icon' clock and normal clocks
			//
			clock.setAttribute("context", "foxclocksoverlay-clock-context");
			clock.setAttribute("tooltip", "foxclocksoverlay-clock-tooltip");
			
			if (previousClock)
			{
				if (previousClock.nextSibling != null)
					clockContainer.insertBefore(clock, previousClock.nextSibling);
				else
					clockContainer.appendChild(clock);
			}
			else
			{				
				// AFM - first clock going in to the clockContainer
				//
				if (this.clockContainerType == "fc-clock-containertype-statusbar")
				{
					var statusbarIndex = null;
					
					if (clocksPosnRelative == "fc-clock-position-left")
						statusbarIndex = 0;
					else if (clocksPosnRelative == "fc-clock-position-right")
						statusbarIndex = clockContainer.childNodes.length;
					else
						statusbarIndex = Number(clocksPosnRelative);

					if (clockContainer.childNodes.length == 0 || statusbarIndex > clockContainer.childNodes.length - 1)
						clockContainer.appendChild(clock);
					else
						clockContainer.insertBefore(clock, clockContainer.childNodes[statusbarIndex]);
				}
				else //  "fc-clock-containertype-toolbar" or anything else
				{
					clockContainer.appendChild(clock);
				}
			}
			
			previousClock = clock;
		}
		
		if (gridRows.childNodes.length == 0)
		{
			// AFM - generate the tool tip grid row when there are no locations to be displayed
			// in the tooltip
			//
			var gridRow = document.createElement("row");
			gridRow.setAttribute("value", "fc-nolocs-row");
			
			var label = document.createElement("label");
			label.setAttribute("value", document.getElementById("foxclocksoverlay-nolocs-label").getAttribute("label"));
			label.setAttribute("disabled", "true");
			
			gridRow.appendChild(label);
			// AFM - don't add second column. Naughty but seems to make it look nicer...
			// gridRow.appendChild(document.createElement("label"));

			gridRows.appendChild(gridRow);
		}
		
		this.setClocksVisibility();
			
		fc_gLogger.log("-FoxClocks_Overlay::createClocks(): arrayLength is " + this.locationArray.length);
	},
	
	// ====================================================================================
	openFoxClocksWindow : function() { return FoxClocks_openWindow("chrome://foxclocks/content/foxclocks.xul", "", "chrome,centerscreen,resizable=yes"); },
	openOptionsWindow : function() { FoxClocks_openWindow("chrome://foxclocks/content/options.xul", "", "chrome,centerscreen,resizable=no"); },
	openAboutWindow : function() { FoxClocks_openWindow("chrome://foxclocks/content/about.xul", "", "chrome,modal,centerscreen,resizable=no"); },
	getClockContainer : function() { return document.getElementById(this.clockContainerType == "fc-clock-containertype-statusbar" ? "status-bar" : "foxclocksoverlay-toolbaritem-box"); },
	getClockXULType : function() { return this.clockContainerType == "fc-clock-containertype-statusbar" ? "statusbarpanel" : "label"; },
	getClockXULAttributeType : function() { return this.clockContainerType == "fc-clock-containertype-statusbar" ? "label" : "value"; },
	
	// ====================================================================================
	showClocksToolbarButton : function (truth)
	{
		fc_gLogger.log("+FoxClocks_Overlay::showClocksToolbarButton()");

		// AFM - can't make the button a data member - changes when it's added/removed from the palette
		//
		var toolbarButton = document.getElementById("foxclocksoverlay-toolbarbutton-clocks");
		
		if (toolbarButton)
			toolbarButton.setAttribute("hidden", (truth ? "false" : "true"));

		// AFM - show/hide the button on the palette. Don't understand this. I thought the above was sufficient, but doesn't always work. Also, have to use wrapper-...:
		// Even though the DOM Inspector reports that foxclocksoverlay-toolbar[item|button]-clocks exist in the customize window, getElementById() fails...
		//
		var paletteItem = this.hCustomizeToolbarWindow ? this.hCustomizeToolbarWindow.document.getElementById("wrapper-foxclocksoverlay-toolbaritem-clocks") : null;
		
		if (paletteItem)
		{
			paletteItem.setAttribute("hidden", (truth ? "false" : "true"));
			// fc_gLogger.log("FoxClocks_Overlay::showClocksToolbarButton(): paletteItem: hidden: " + paletteItem.getAttribute("hidden"));
		}	
		else
		{
			// fc_gLogger.log("FoxClocks_Overlay::showClocksToolbarButton(): don't have paletteItem: cust window exists: " + (this.hCustomizeToolbarWindow != null));
			
			// if (this.hCustomizeToolbarWindow != null)
			// {
				// var x = this.hCustomizeToolbarWindow.document.getElementById("wrapper-foxclocksoverlay-toolbaritem-clocks");
				// fc_gLogger.log("FoxClocks_Overlay::showClocksToolbarButton(): cust window exists: x: " + (x != null));
			// }
		}
		
		fc_gLogger.log("-FoxClocks_Overlay::showClocksToolbarButton()");
	},
	
	// ====================================================================================
	// AFM - legacy code to support CuteMenus: http://www.extensionsmirror.nl/index.php?showtopic=4360
	//
	onMenuItemCmd : function(event) { this.openFoxClocksWindow(); },
	
	// ====================================================================================
	// AFM - implementing nsIWebProgressListener
	// see http://kb.mozillazine.org/Progress_listeners
	// ====================================================================================
	onStateChangex: function(aProgress, aRequest, aFlag, aStatus)
	{
		// AFM - vain attempt to stop the browser from coming to the front and hiding the FoxClocks
		// window, in the case that both the homepage and FoxClocks window were resquested on the
		// welcome screen. The below doesn't work, but using the timer (commented out) seems to work.
		// However, we have no idea how long the load took, and it may be that the user has already inspected/
		// closed/whatever the various windows, so we don't want to be messing with them
		//
		if (aFlag & (Components.interfaces.nsIWebProgressListener.STATE_STOP | Components.interfaces.nsIWebProgressListener.STATE_IS_WINDOW))
		{
			fc_gLogger.log("+FoxClocks_Overlay::onStateChange()");
		
			var mediatorService = Components.classes["@mozilla.org/appshell/window-mediator;1"].getService(Components.interfaces.nsIWindowMediator);
			var mrWindow = mediatorService.getMostRecentWindow("navigator:browser");
			
			// setTimeout(this._blur, 0, self);
			mrWindow.blur();
			
			var browser = mrWindow.document.getElementById("content");
			browser.removeProgressListener(this);
			
			fc_gLogger.log("-FoxClocks_Overlay::onStateChange()");
		}
		
		return 0;
	},
	
	// ====================================================================================
	onLocationChangex: function(aProgress, aRequest, aURI) { return 0; },
	onProgressChangex: function() { return 0; },
	onStatusChangex: function() { return 0; },
	onSecurityChangex: function() { return 0; },
	onLinkIconAvailablex: function() { return 0; }	
}