// FoxClocks extension for Mozilla Firefox/Thunderbird/Sunbird
// Copyright (C) 2006 Andy McDonald / www.stemhaus.com
// For licensing terms, please refer to readme.txt in this extension's '.xpi'
// package or its installation directory on your computer.	

// AFM - TODO: to components/
//
	
// ====================================================================================
function FoxClocks_TimeFormatter(formatString)
{	
	FoxClocks_TimeFormatter.init(); // static init
	this.currentFormatArray = formatString ? FoxClocks_TimeFormatter.createFormatArray(formatString) : new Array();
}

// ====================================================================================
FoxClocks_TimeFormatter.prototype =
{
	// ====================================================================================
	getTimeString : function(location, date)
	{
		return FoxClocks_TimeFormatter._getTimeString(location, date, this.currentFormatArray);
	},
		
	// ====================================================================================
	setTimeFormat : function(formatString)
	{
		this.currentFormatArray = FoxClocks_TimeFormatter.createFormatArray(formatString);
	}	
}

// ====================================================================================
FoxClocks_TimeFormatter.init = function()
{
	if (typeof(FoxClocks_TimeFormatter.doneInit) != "undefined")
		return;
		
	fc_gLogger.log("+FoxClocks_TimeFormatter::init()");
	
	FoxClocks_TimeFormatter.doneInit = true;
	
	var hFoxClocksBundle = document.getElementById("foxclocks-bundle");
	
	FoxClocks_TimeFormatter.formatters = new Array
	(
		new FC_Format12Hour1(hFoxClocksBundle),
		new FC_Format12Hour2(hFoxClocksBundle),
		new FC_Format24Hour1(hFoxClocksBundle),
		new FC_Format24Hour2(hFoxClocksBundle),
		new FC_FormatMins(hFoxClocksBundle),
		new FC_FormatSecs(hFoxClocksBundle),
		new FC_FormatAmLower(hFoxClocksBundle),
		new FC_FormatAmUpper(hFoxClocksBundle),
		new FC_FormatYear4(hFoxClocksBundle),
		new FC_FormatYear2(hFoxClocksBundle),
		new FC_FormatMonthNum1(hFoxClocksBundle),
		new FC_FormatMonthNum2(hFoxClocksBundle),
		new FC_FormatMonthNameLong(hFoxClocksBundle),
		new FC_FormatMonthNameShort(hFoxClocksBundle),
		new FC_FormatDayNum1(hFoxClocksBundle),
		new FC_FormatDayNum2(hFoxClocksBundle),
		new FC_FormatDayNameLong(hFoxClocksBundle),
		new FC_FormatDayNameShort(hFoxClocksBundle),
		new FC_FormatDayOrdinal(hFoxClocksBundle),
		new FC_FormatDayOfYear(hFoxClocksBundle),
		new FC_FormatLocation(hFoxClocksBundle),
		new FC_FormatOffset(hFoxClocksBundle),
		new FC_FormatZoneName(hFoxClocksBundle)
	);
	
	// AFM - TODO - remove try-catch when property translated
	//
	try
	{
		FoxClocks_TimeFormatter.formatters.push(new FC_FormatDSTIndicator(hFoxClocksBundle));
	}
	catch (ex)
	{
		fc_gLogger.log("FoxClocks_TimeFormatter::init(): DST indicator not supported: " +ex);
	}
	
	var formatsRegexpString = "";	
	var isFirstIndex = true;
	
	for (var i=0; i < FoxClocks_TimeFormatter.formatters.length; i++)
	{
		if (isFirstIndex)
			isFirstIndex = false;
		else
			formatsRegexpString += "|";
	
		// AFM - not using the format regexps really, but keeping them around for future use
		//
		formatsRegexpString += FoxClocks_TimeFormatter.formatters[i].placeholderRegexp.source;
	}
	
	FoxClocks_TimeFormatter.formatsRegexp = new RegExp(formatsRegexpString, "g");
	
	fc_gLogger.log("-FoxClocks_TimeFormatter::init()");
}

// ====================================================================================
FoxClocks_TimeFormatter.getTimeString = function(location, date, formatString)
{	
	var formatArray = FoxClocks_TimeFormatter.createFormatArray(formatString);
	return FoxClocks_TimeFormatter._getTimeString(location, date, formatArray);
}

// ====================================================================================
FoxClocks_TimeFormatter._getTimeString = function(location, date, formatArray)
{	
	var displayDate = location ? location.zone.currDisplayDate(date) : date;
	var timeString = "";
	
	for (var i=0; i < formatArray.length; i++)
	{
		// AFM - pass in the original date for things like FC_FormatZoneName.dateToZoneName
		//
		timeString += formatArray[i].dateToText(date, displayDate, location);
	}
	
	return timeString;
}
	
// ====================================================================================
FoxClocks_TimeFormatter.createFormatArray = function(formatString)
{
	FoxClocks_TimeFormatter.init(); // static init
	
	// AFM - anon function is not in the scope of the zone manager
	//
	var tmpCurrentFormat = new Array();
	var tmpFormatObjects = new Array();
	
	for (var i=0; i < FoxClocks_TimeFormatter.formatters.length; i++)
	{
		tmpFormatObjects.push(FoxClocks_TimeFormatter.formatters[i]);
	}
	
	// AFM - we make a reasonable (??) assumption that matches are made in the order in which they occur in the text...
	// So anyway, this is ugly and grim, and we're not really doing any replacing; we just want the anon function executed on
	// each match, and I'm guessing it's better than regexp.exec()...
	// Frankly, this is all nonsense...
	//
	var nextMatchIndex = 0;
	
	formatString.replace(FoxClocks_TimeFormatter.formatsRegexp,
	
		function (str, p1, p2, offset, s)
		{
			if (nextMatchIndex < p1)
			{
				var unmatchedText = formatString.substr(nextMatchIndex, (p1 - nextMatchIndex))
				tmpCurrentFormat.push(new FC_FormatText(unmatchedText));
			}
	
			nextMatchIndex = p1 + str.length;
			
			for (var j = 0; j < tmpFormatObjects.length; j++)
			{
				if (str == tmpFormatObjects[j].placeholderRegexp.source)
				{
					tmpCurrentFormat.push(tmpFormatObjects[j]);
					break;
				}
			}
		}
	);
	
	// AFM - check for text after the last match
	//
	if (nextMatchIndex <= formatString.length - 1)
	{
		var text = formatString.substr(nextMatchIndex)
		tmpCurrentFormat.push(new FC_FormatText(text));
	}
	
	return tmpCurrentFormat;
}

// ====================================================================================
function FC_FormatText(text)
{
	if (typeof(FC_FormatText.prototype.dateToText) == "undefined")
	{
		FC_FormatText.prototype.dateToText = function (date, displayDate) { return this.text; }
		FC_FormatText.prototype.placeholderRegexp = null;
	}

	this.text = text;
}

// ====================================================================================
function FC_Format12Hour1(hBundle)
{
	if (typeof(FC_Format12Hour1.prototype.dateToText) == "undefined")
	{
		FC_Format12Hour1.prototype.dateToText = function (date, displayDate)
		{ return fc_gUtils.as12hr(displayDate.getUTCHours()); }
	}
	
	this.placeholderRegexp = new RegExp(hBundle.getString("options.format.custom.hours.12.1.value"), "g");
}

// ====================================================================================
function FC_Format12Hour2(hBundle)
{
	if (typeof(FC_Format12Hour2.prototype.dateToText) == "undefined")
	{
		FC_Format12Hour2.prototype.dateToText = function (date, displayDate)
		{ return fc_gUtils.asTwoDigit(fc_gUtils.as12hr(displayDate.getUTCHours()));}
	}
	
	this.placeholderRegexp = new RegExp(hBundle.getString("options.format.custom.hours.12.2.value"), "g");
}

// ====================================================================================
function FC_Format24Hour1(hBundle)
{
	if (typeof(FC_Format24Hour1.prototype.dateToText) == "undefined")
	{
		FC_Format24Hour1.prototype.dateToText = function(date, displayDate)
		{ return displayDate.getUTCHours(); }
	}
	
	this.placeholderRegexp = new RegExp(hBundle.getString("options.format.custom.hours.24.1.value"), "g");
}

// ====================================================================================
function FC_Format24Hour2(hBundle)
{
	if (typeof(FC_Format24Hour2.prototype.dateToText) == "undefined")
	{
		FC_Format24Hour2.prototype.dateToText = function (date, displayDate)
		{ return fc_gUtils.asTwoDigit(displayDate.getUTCHours()); }
	}
	
	this.placeholderRegexp = new RegExp(hBundle.getString("options.format.custom.hours.24.2.value"), "g");
}

// ====================================================================================
function FC_FormatMins(hBundle)
{
	if (typeof(FC_FormatMins.prototype.dateToText) == "undefined")
	{
		FC_FormatMins.prototype.dateToText = function (date, displayDate)
		{ return fc_gUtils.asTwoDigit(displayDate.getUTCMinutes()); }
	}
	
	this.placeholderRegexp = new RegExp(hBundle.getString("options.format.custom.mins.value"), "g");
}

// ====================================================================================
function FC_FormatSecs(hBundle)
{
	if (typeof(FC_FormatSecs.prototype.dateToText) == "undefined")
	{
		FC_FormatSecs.prototype.dateToText = function (date, displayDate)
		{ return fc_gUtils.asTwoDigit(displayDate.getUTCSeconds()); }
	}
	
	this.placeholderRegexp = new RegExp(hBundle.getString("options.format.custom.secs.value"), "g");
}

// ====================================================================================
function FC_FormatAmLower(hBundle)
{
	if (typeof(FC_FormatAmLower.prototype.dateToText) == "undefined")
	{
		FC_FormatAmLower.prototype.dateToText = function (date, displayDate)
		{ return displayDate.getUTCHours() < 12 ? this.amlower : this.pmlower; }
	}
	
	this.placeholderRegexp = new RegExp(hBundle.getString("options.format.custom.am.lower.value"), "g");
	this.amlower = hBundle.getString("misc.am.lower");
	this.pmlower = hBundle.getString("misc.pm.lower");
}

// ====================================================================================
function FC_FormatAmUpper(hBundle)
{
	if (typeof(FC_FormatAmUpper.prototype.dateToText) == "undefined")
	{
		FC_FormatAmUpper.prototype.dateToText = function (date, displayDate)
		{ return displayDate.getUTCHours() < 12 ? this.amupper : this.pmupper; }
	}
	
	this.placeholderRegexp = new RegExp(hBundle.getString("options.format.custom.am.upper.value"), "g");
	this.amupper = hBundle.getString("misc.am.upper");
	this.pmupper = hBundle.getString("misc.pm.upper");
}	

// ====================================================================================
function FC_FormatYear4(hBundle)
{
	if (typeof(FC_FormatYear4.prototype.dateToText) == "undefined")
	{
		FC_FormatYear4.prototype.dateToText = function (date, displayDate)
		{ return displayDate.getUTCFullYear(); }
	}
	
	this.placeholderRegexp = new RegExp(hBundle.getString("options.format.custom.year.4.value"), "g");
}

// ====================================================================================
function FC_FormatYear2(hBundle)
{
	if (typeof(FC_FormatYear2.prototype.dateToText) == "undefined")
	{
		FC_FormatYear2.prototype.dateToText = function (date, displayDate)
		{ return displayDate.getUTCFullYear().toString().substr(2, 2); }
	}		
	
	this.placeholderRegexp = new RegExp(hBundle.getString("options.format.custom.year.2.value"), "g");
}

// ====================================================================================
function FC_FormatMonthNum1(hBundle)
{
	if (typeof(FC_FormatMonthNum1.prototype.dateToText) == "undefined")
	{
		FC_FormatMonthNum1.prototype.dateToText = function (date, displayDate)
		{ return displayDate.getUTCMonth() + 1; }
	}
	
	this.placeholderRegexp = new RegExp(hBundle.getString("options.format.custom.month.num.1.value"), "g");
}

// ====================================================================================
function FC_FormatMonthNum2(hBundle)
{
	if (typeof(FC_FormatMonthNum2.prototype.dateToText) == "undefined")
	{
		FC_FormatMonthNum2.prototype.dateToText = function (date, displayDate)
		{ return fc_gUtils.asTwoDigit(displayDate.getUTCMonth() + 1); }
	}
	
	this.placeholderRegexp = new RegExp(hBundle.getString("options.format.custom.month.num.2.value"), "g");
}

// ====================================================================================
function FC_FormatMonthNameLong(hBundle)
{
	if (typeof(FC_FormatMonthNameLong.prototype.dateToText) == "undefined")
	{
		FC_FormatMonthNameLong.prototype.dateToText = function (date, displayDate)
		{ return this.listMonthsLong[displayDate.getUTCMonth()]; }
	}
	
	this.placeholderRegexp = new RegExp(hBundle.getString("options.format.custom.month.name.long.value"), "g");
	this.listMonthsLong = new Array(); this.listMonthsLong = hBundle.getString("misc.list.month.name.long").split(",");
}

// ====================================================================================
function FC_FormatMonthNameShort(hBundle)
{
	if (typeof(FC_FormatMonthNameShort.prototype.dateToText) == "undefined")
	{
		FC_FormatMonthNameShort.prototype.dateToText = function (date, displayDate)
		{ return this.listMonthsShort[displayDate.getUTCMonth()]; }
	}
	
	this.placeholderRegexp = new RegExp(hBundle.getString("options.format.custom.month.name.short.value"), "g");
	this.listMonthsShort = new Array(); this.listMonthsShort = hBundle.getString("misc.list.month.name.short").split(",");
}

// ====================================================================================
function FC_FormatDayNum1(hBundle)
{
	if (typeof(FC_FormatDayNum1.prototype.dateToText) == "undefined")
	{
		FC_FormatDayNum1.prototype.dateToText = function (date, displayDate)
		{ return displayDate.getUTCDate(); }
	}
	
	this.placeholderRegexp = new RegExp(hBundle.getString("options.format.custom.day.num.1.value"), "g");
}

// ====================================================================================
function FC_FormatDayNum2(hBundle)
{
	if (typeof(FC_FormatDayNum2.prototype.dateToText) == "undefined")
	{
		FC_FormatDayNum2.prototype.dateToText = function (date, displayDate)
		{ return fc_gUtils.asTwoDigit(displayDate.getUTCDate()); }
	}
	
	this.placeholderRegexp = new RegExp(hBundle.getString("options.format.custom.day.num.2.value"), "g");
}

// ====================================================================================
function FC_FormatDayNameLong(hBundle)
{
	if (typeof(FC_FormatDayNameLong.prototype.dateToText) == "undefined")
	{
		FC_FormatDayNameLong.prototype.dateToText = function (date, displayDate)
		{ return this.listDaysLong[displayDate.getUTCDay()]; }
	}
	
	this.placeholderRegexp = new RegExp(hBundle.getString("options.format.custom.day.name.long.value"), "g");
	this.listDaysLong = new Array(); this.listDaysLong = hBundle.getString("misc.list.day.name.long").split(",");	
}

// ====================================================================================
function FC_FormatDayNameShort(hBundle)
{
	if (typeof(FC_FormatDayNameShort.prototype.dateToText) == "undefined")
	{
		FC_FormatDayNameShort.prototype.dateToText = function (date, displayDate)
		{ return this.listDaysShort[displayDate.getUTCDay()]; }
	}
	
	this.placeholderRegexp = new RegExp(hBundle.getString("options.format.custom.day.name.short.value"), "g");
	this.listDaysShort = new Array(); this.listDaysShort = hBundle.getString("misc.list.day.name.short").split(",");
}

// ====================================================================================
function FC_FormatDayOrdinal(hBundle)
{
	if (typeof(FC_FormatDayOrdinal.prototype.dateToText) == "undefined")
	{
		FC_FormatDayOrdinal.prototype.dateToText = function (date, displayDate)
		{ return this.listDaysOrdinal[displayDate.getUTCDate() - 1]; }
	}
	
	this.placeholderRegexp = new RegExp(hBundle.getString("options.format.custom.day.ordinal.value"), "g");
	this.listDaysOrdinal = new Array(); this.listDaysOrdinal = hBundle.getString("misc.list.day.ordinal").split(",");
}

// ====================================================================================
function FC_FormatDayOfYear(hBundle)
{
	if (typeof(FC_FormatDayOfYear.prototype.dateToText) == "undefined")
	{
		FC_FormatDayOfYear.DAYS_IN_MONTH_NORMAL = new Array(31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31);
		FC_FormatDayOfYear.DAYS_IN_MONTH_LEAP = new Array(31, 29, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31);
		FC_FormatDayOfYear.prototype.dateToText = function (date, displayDate)
		{
			var day = displayDate.getUTCDate();
			var month = displayDate.getUTCMonth();
			var year = displayDate.getUTCFullYear();
			
			var isLeapYear = year % 4 == 0 && (year % 100 != 0 || year % 400 != 0);
			var monthArray = isLeapYear ? FC_FormatDayOfYear.DAYS_IN_MONTH_LEAP :
									FC_FormatDayOfYear.DAYS_IN_MONTH_NORMAL;
		
			var dayOfYear = 0;
			for (var i=0; i < month; i++)
			{
				dayOfYear += monthArray[i];
			}
			dayOfYear += day;
			
			return dayOfYear;
		}
	}
	
	this.placeholderRegexp = new RegExp(hBundle.getString("options.format.custom.day.ofyear.value"), "g");
}

// ====================================================================================
function FC_FormatLocation(hBundle)
{
	if (typeof(FC_FormatLocation.prototype.dateToText) == "undefined")
	{
		FC_FormatLocation.prototype.dateToText = function (date, displayDate, location)
		{ if (location) return location.name; else return this.defaultLoc; }
	}
	
	this.defaultLoc = hBundle.getString("misc.default.location");
	this.placeholderRegexp = new RegExp(hBundle.getString("options.format.custom.other.location.value"), "g");
}

// ====================================================================================
function FC_FormatOffset(hBundle)
{
	if (typeof(FC_FormatOffset.prototype.dateToText) == "undefined")
	{
		FC_FormatOffset.prototype.dateToText = function (date, displayDate, location)
		{ if (location) return location.zone.currOffsetString(date); else return this.defaultOffset; }
	}

	this.defaultOffset = hBundle.getString("misc.default.offset");
	this.placeholderRegexp = new RegExp(hBundle.getString("options.format.custom.other.offset.value"), "g");
}

// ====================================================================================
function FC_FormatZoneName(hBundle)
{
	if (typeof(FC_FormatZoneName.prototype.dateToText) == "undefined")
	{
		FC_FormatZoneName.prototype.dateToText = function (date, displayDate, location)
		{ if (location) return location.zone.currName(date); else return this.defaultZoneName; }
	}
	
	this.defaultZoneName = hBundle.getString("misc.default.zonename");
	this.placeholderRegexp = new RegExp(hBundle.getString("options.format.custom.other.zone.value"), "g");
}

// ====================================================================================
function FC_FormatDSTIndicator(hBundle)
{
	if (typeof(FC_FormatDSTIndicator.prototype.dateToText) == "undefined")
	{
		FC_FormatDSTIndicator.prototype.dateToText = function (date, displayDate, location)
		{ if (location) return (location.zone.isDST(date) ? "*" : ""); else return this.defaultDSTIndicator; }
	}
	
	this.defaultDSTIndicator = hBundle.getString("misc.default.dstindic");
	this.placeholderRegexp = new RegExp(hBundle.getString("options.format.custom.other.dstindic.value"), "g");
}